#include <console/console.h>
#include <utility/memory.h>
#include <memory/memory.h>
#include <libc/stdlib.h>
#include <debugger/pong.h>

Bool
pong_object_create (PongObject * object,
                    int xPosition,
                    int yPosition,
                    int xVelocity,
                    int yVelocity,
                    Size width,
                    Size height,
                    CanvasColor color)
{
     object -> xPosition = xPosition;
     object -> yPosition = yPosition;
     object -> xVelocity = xVelocity;
     object -> yVelocity = yVelocity;
     object -> width     = width;
     object -> height    = height;
     object -> color     = color;

     return TRUE;
}

Bool
pong_object_intersect (const PongObject * object,
                       int xStart,
                       int yStart,
                       int xDelta,
                       int yDelta)
{
    int collideStart,
        collideEnd,
        objectStart,
        objectEnd;
        
       
    if (xDelta == 0)
    {
       if (xStart < object -> xPosition ||
           xStart >= object -> xPosition + object -> width)
         return FALSE;

       collideStart = yStart;
       collideEnd   = yStart + yDelta;
       objectStart  = object -> yPosition;
       objectEnd    = object -> yPosition + object -> height;
    }
    else
    if (yDelta == 0)
    {
       if (yStart < object -> yPosition ||
           yStart >= object -> yPosition + object -> height)
         return FALSE;

       collideStart = xStart;
       collideEnd   = xStart + xDelta;
       objectStart  = object -> xPosition;
       objectEnd    = object -> xPosition + object -> width;
    }
    else
    {
       collideStart = yDelta * (object -> xPosition - xStart) / xDelta + yStart,
       collideEnd   = yDelta * (object -> xPosition + object -> width - xStart) / xDelta + yStart;
       objectStart  = object -> yPosition;
       objectEnd    = object -> yPosition + object -> height;
    }

    if (collideStart >= objectStart &&
        collideStart < objectEnd)
      return TRUE;

    if (collideEnd >= objectStart &&
        collideEnd < objectEnd)
      return TRUE;

    if (collideStart < objectStart &&
        collideEnd > objectEnd)
      return TRUE;

    if (collideEnd < objectStart &&
        collideStart > objectEnd)
      return TRUE;

    return FALSE;
}

Bool
pong_object_collision (const PongObject * mobile,
                       const PongObject * obstacle)
{
    return pong_object_intersect (obstacle, 
                                  mobile -> xPosition,
                                  mobile -> yPosition,
                                  mobile -> xVelocity,
                                  mobile -> yVelocity) ||
           pong_object_intersect (obstacle,
                                  mobile -> xPosition + mobile -> width,
                                  mobile -> yPosition,
                                  mobile -> xVelocity,
                                  mobile -> yVelocity) ||
           pong_object_intersect (obstacle,
                                  mobile -> xPosition,
                                  mobile -> yPosition + mobile -> height,
                                  mobile -> xVelocity,
                                  mobile -> yVelocity) ||
           pong_object_intersect (obstacle,
                                  mobile -> xPosition + mobile -> width,
                                  mobile -> yPosition + mobile -> height,
                                  mobile -> xVelocity,
                                  mobile -> yVelocity) ||
           pong_object_intersect (mobile,
                                  obstacle -> xPosition,
                                  obstacle -> yPosition,
                                  - mobile -> xVelocity,
                                  - mobile -> yVelocity) ||
           pong_object_intersect (mobile,
                                  obstacle -> xPosition + obstacle -> width,
                                  obstacle -> yPosition,
                                  - mobile -> xVelocity,
                                  - mobile -> yVelocity) ||
           pong_object_intersect (mobile,
                                  obstacle -> xPosition,
                                  obstacle -> yPosition + obstacle -> height,
                                  - mobile -> xVelocity,
                                  - mobile -> yVelocity) ||
           pong_object_intersect (mobile,
                                  obstacle -> xPosition + obstacle -> width,
                                  obstacle -> yPosition + obstacle -> height,
                                  - mobile -> xVelocity,
                                  - mobile -> yVelocity);

}

void
pong_object_bounce (PongObject * object)
{
    int velocity = (object -> xVelocity > 0 ? object -> xVelocity : - object -> xVelocity) +
                   (object -> yVelocity > 0 ? object -> yVelocity : - object -> yVelocity);

    object -> xVelocity = (int) ((velocity + 1.0) * rand () / (RAND_MAX + 1.0));
    object -> yVelocity = velocity - object -> xVelocity;

    object -> xVelocity *= -1 + 2 * (int) (2.0 * rand () / (RAND_MAX + 1.0));
    object -> yVelocity *= -1 + 2 * (int) (2.0 * rand () / (RAND_MAX + 1.0));
}

void
pong_object_move (PongObject * object)
{
    object -> xPosition += object -> xVelocity;
    object -> yPosition += object -> yVelocity;
}

Bool
pong_arena_intersect (PongArena * arena, 
                      int xStart, 
                      int yStart, 
                      int xDelta,
                      int yDelta)
{
    if (xStart + xDelta < 0 ||
        xStart + xDelta >= arena -> arena.width ||
        yStart + yDelta < 0 ||
        yStart + yDelta >= arena -> arena.height)
      return TRUE;

    return FALSE;
}

PongObject *
pong_arena_collision (PongArena * arena, const PongObject * mobile)
{
    PongObject * ball;

    if (pong_arena_intersect (arena,
                              mobile -> xPosition,
                              mobile -> yPosition,
                              mobile -> xVelocity,
                              mobile -> yVelocity) ||
        pong_arena_intersect (arena,
                              mobile -> xPosition + mobile -> width,
                              mobile -> yPosition,
                              mobile -> xVelocity,
                              mobile -> yVelocity) ||
        pong_arena_intersect (arena,
                              mobile -> xPosition,
                              mobile -> yPosition + mobile -> height,
                              mobile -> xVelocity,
                              mobile -> yVelocity) ||
       pong_arena_intersect (arena,
                              mobile -> xPosition + mobile -> width,
                              mobile -> yPosition + mobile -> height,
                              mobile -> xVelocity,
                              mobile -> yVelocity))
      return & arena -> arena;

    if (mobile != & arena -> paddle &&
        pong_object_collision (mobile, & arena -> paddle) == TRUE)
      return & arena -> paddle;

    for (ball = arena -> balls;
         ball < & arena -> balls [arena -> ballCount];
         ++ ball)
    {
       if (ball == mobile)
         continue;

       if (pong_object_collision (mobile, ball) == TRUE)
         return ball;
    }
    
    return NULL;
}

void
pong_arena_motion (PongArena * arena)
{
    PongObject * ball,
               * obstacle;

    obstacle = pong_arena_collision (arena, & arena -> paddle);
    
    if (obstacle != NULL)
    {
       if (obstacle != & arena -> arena)
         arena -> paddle.color = 1 + (CanvasColor) (15.0 * rand () / (RAND_MAX + 1.0));

       pong_object_bounce (obstacle);
    }    
    else
      pong_object_move (& arena -> paddle);

    for (ball = arena -> balls;
         ball < & arena -> balls [arena -> ballCount];
         ++ ball)
    {
       obstacle = pong_arena_collision (arena, ball);
       
       if (obstacle != NULL)
         pong_object_bounce (ball);
       else
         pong_object_move (ball);
    }
}

static void
frameTimerHandlerFunction (Timer * timer, void * data)
{
     PongArena * arena = (PongArena *) data;

     semaphore_up (& arena -> frameSemaphore);
}

static void
keyboardEventHandlerFunction (Keyboard * keyboard, const KeyEvent * event, void * data)
{
     PongArena * arena = (PongArena *) data;
     PongObject * newBalls;

     if (event -> state == KEY_STATE_RELEASED)
       return;

     mutex_lock (& arena -> arenaMutex);

     switch (event -> key)
     {
     case 'q':
     case 'Q':
        arena -> quitRequested = TRUE;

        break;

     case 'r':
     case 'R':
        if (arena -> ballCount > 1)
          -- arena -> ballCount;

        break;

     case 'a':
     case 'A':
        newBalls = (PongObject *) memory_resize (arena -> balls, (arena -> ballCount + 1) * sizeof (PongObject));

        if (newBalls == NULL)
          return;

        arena -> balls = newBalls;

        ++ arena -> ballCount;

        while (newBalls < & arena -> balls [arena -> ballCount])
        {
           pong_object_create (newBalls,
                               ((newBalls - arena -> balls) * 2 * arena -> balls -> width) % 
                                 arena -> arena.width,
                               arena -> balls -> height + (((newBalls - arena -> balls) * 2 * arena -> balls -> width) /
                                  arena -> arena.width) *
                                    2 * arena -> balls -> height,
                               1 + (int) ((arena -> balls -> width / 5.0 + 1.0) * rand () / (RAND_MAX + 1.0)),
                               1 + (int) ((arena -> balls -> width / 5.0 + 1.0) * rand () / (RAND_MAX + 1.0)),
                               arena -> balls -> width,
                               arena -> balls -> height,
                               1 + (CanvasColor) (15.0 * rand () / (RAND_MAX + 1.0)));

           ++ newBalls;
        }
       
        break;
 
     case KEY_CURSOR_UP:
     case KEY_KEYPAD_UP:
        arena -> paddle.xVelocity = 0;
        arena -> paddle.yVelocity = - arena -> arena.height / 20;

        break;

     case KEY_CURSOR_DOWN:
     case KEY_KEYPAD_DOWN:
        arena -> paddle.xVelocity = 0;
        arena -> paddle.yVelocity = arena -> arena.height / 20;

        break;

     case KEY_CURSOR_LEFT:
     case KEY_KEYPAD_LEFT:
        arena -> paddle.xVelocity = - arena -> arena.width / 20;
        arena -> paddle.yVelocity = 0;

        break;

     case KEY_CURSOR_RIGHT:
     case KEY_KEYPAD_RIGHT:
        arena -> paddle.xVelocity = arena -> arena.width / 20;
        arena -> paddle.yVelocity = 0;

        break;
     }

     mutex_unlock (& arena -> arenaMutex);
}

Bool
pong_arena_create (PongArena * arena, 
                   int arenaWidth, 
                   int arenaHeight,
                   int paddleWidth,
                   int paddleHeight,
                   int ballWidth,
                   int ballHeight,
                   Size ballCount)
{
     PongObject * ball;
     TimeOffset frameTime;

     arena -> ballCount     = ballCount;
     arena -> quitRequested = FALSE;

     arena -> balls = (PongObject *) memory_allocate (ballCount * sizeof (PongObject));

     if (arena -> balls == NULL)
       return FALSE;

     for (ball = arena -> balls;
          ball < & arena -> balls [arena -> ballCount];
          ++ ball)
     {
        pong_object_create (ball,
                            ((ball - arena -> balls) * 2 * ballWidth) % arenaWidth,
                            ballHeight + (((ball - arena -> balls) * 2 * ballWidth) / arenaWidth) * 2 * ballHeight,
                            1 + (int) ((ballWidth / 5.0 + 1.0) * rand () / (RAND_MAX + 1.0)),
                            1 + (int) ((ballHeight / 5.0 + 1.0) * rand () / (RAND_MAX + 1.0)),
                            ballWidth,
                            ballHeight,
                            1 + (CanvasColor) (15.0 * rand () / (RAND_MAX + 1.0))); 
     }
     
     pong_object_create (& arena -> arena,
                         0,
                         0,
                         0,
                         0,
                         arenaWidth,
                         arenaHeight,
                         0);
     
     pong_object_create (& arena -> paddle,
                         (arenaWidth - paddleWidth) / 2,
                         arenaHeight - 2 * paddleHeight,
                         0,
                         0,
                         paddleWidth,
                         paddleHeight,
                         1 + (CanvasColor) (15.0 * rand () / (RAND_MAX + 1.0)));

     semaphore_create (& arena -> frameSemaphore, 0);
     mutex_create (& arena -> arenaMutex);

     frameTime.seconds     = 0;
     frameTime.nanoSeconds = NANOSECONDS_PER_SECOND / 10;

     timer_create (& arena -> frameTimer, & frameTime, frameTimerHandlerFunction, arena, TRUE);

     if (vga_context_create (& arena -> vgaContext, 
                             VGA_MODE_IMAGE_320x200x8,
                             NULL,
                             NULL,
                             NULL,
                             NULL) == FALSE)
     {
        memory_free (arena -> balls);
        timer_destroy (& arena -> frameTimer);
        semaphore_destroy (& arena -> frameSemaphore);
        mutex_destroy (& arena -> arenaMutex);

        return FALSE;
     }

     if (keyboard_create (& arena -> keyboardContext,
                          NULL,
                          NULL,
                          NULL,
                          NULL,
                          NULL,
                          keyboardEventHandlerFunction,
                          arena) == FALSE)
     {
        vga_context_destroy (& arena -> vgaContext);
        memory_free (arena -> balls);
        timer_destroy (& arena -> frameTimer);
        semaphore_destroy (& arena -> frameSemaphore);
        mutex_destroy (& arena -> arenaMutex);

        return FALSE;
     }

     if (canvas_create (& arena -> canvas, 320, 200, 8) == FALSE)
     {
        keyboard_destroy (& arena -> keyboardContext);
        vga_context_destroy (& arena -> vgaContext);
        memory_free (arena -> balls);
        timer_destroy (& arena -> frameTimer);
        semaphore_destroy (& arena -> frameSemaphore);
        mutex_destroy (& arena -> arenaMutex);
 
        return FALSE;
     }
        
     return TRUE;
}

void
pong_arena_destroy (PongArena * arena)
{
     canvas_destroy (& arena -> canvas);
     keyboard_destroy (& arena -> keyboardContext);
     vga_context_destroy (& arena -> vgaContext);
     memory_free (arena -> balls);
     timer_destroy (& arena -> frameTimer);
     semaphore_destroy (& arena -> frameSemaphore);
     mutex_destroy (& arena -> arenaMutex);
}

void
pong_arena_draw (PongArena * arena)
{
     CanvasArea rectangle;
     CanvasPoint triangle [3];
     PongObject * ball;

     rectangle.x      = (Size) arena -> paddle.xPosition;
     rectangle.y      = (Size) arena -> paddle.yPosition;
     rectangle.width  = (Size) arena -> paddle.width;
     rectangle.height = (Size) arena -> paddle.height;

     canvas_fill (& arena -> canvas, arena -> arena.color);

     canvas_rectangle (& arena -> canvas, arena -> paddle.color, & rectangle);

     triangle [0].x = rectangle.x + rectangle.width / 4;
     triangle [0].y = rectangle.y + rectangle.height * 3 / 4; 
     triangle [1].x = rectangle.x + rectangle.width / 2;
     triangle [1].y = rectangle.y + rectangle.height / 4;
     triangle [2].x = rectangle.x + rectangle.width * 3 / 4;
     triangle [2].y = rectangle.y + rectangle.height * 3 / 4;

     canvas_triangle (& arena -> canvas,
                      1 + ((arena -> paddle.color + 1) % 15),
                      & triangle [0],
                      & triangle [1],
                      & triangle [2]);
    
     for (ball = arena -> balls;
          ball < & arena -> balls [arena -> ballCount];
          ++ ball)
     {
         rectangle.x      = (Size) ball -> xPosition;
         rectangle.y      = (Size) ball -> yPosition;
         rectangle.width  = (Size) ball -> width;
         rectangle.height = (Size) ball -> height;

         canvas_rectangle (& arena -> canvas, ball -> color, & rectangle);
     }

     memory_copy (arena -> vgaContext.videoMemory,
                  arena -> canvas.pixels,
                  arena -> canvas.width * arena -> canvas.height * arena -> canvas.depth / 8);
}

void
pong_arena_play (PongArena * arena)
{
     arena -> quitRequested = FALSE;
    
     vga_context_activate (& arena -> vgaContext);

     pong_arena_draw (arena);
     
     keyboard_activate (& arena -> keyboardContext);

     timer_start (& arena -> frameTimer);

     while (arena -> quitRequested == FALSE &&
            vga_context_current () == & arena -> vgaContext)
     {
         semaphore_down (& arena -> frameSemaphore);

         mutex_lock (& arena -> arenaMutex);

         pong_arena_motion (arena);

         pong_arena_draw (arena);
        
         mutex_unlock (& arena -> arenaMutex);
     }

     timer_stop (& arena -> frameTimer, NULL);
     
     keyboard_activate (NULL);

     console_activate (console_current ());
}

