#ifndef __DEBUGGER_PONG_H__
#define __DEBUGGER_PONG_H__

#include <utility/types.h>
#include <vga/context.h>
#include <kbc/keyboard.h>
#include <canvas/canvas.h>
#include <timer/timer.h>
#include <scheduler/mutex.h>
#include <scheduler/semaphore.h>

typedef struct
{
   int         xPosition;
   int         yPosition;
   int         xVelocity;
   int         yVelocity;
   int         width;
   int         height;
   CanvasColor color;
} PongObject;

typedef struct
{
        PongObject * balls;
        Size         ballCount;
        PongObject   arena;
        PongObject   paddle;
        Canvas       canvas;
        Bool         quitRequested;
        Semaphore    frameSemaphore;
        Timer        frameTimer;
        VGAContext   vgaContext;
        Keyboard     keyboardContext;
        Mutex        arenaMutex;
} PongArena;

extern Bool         pong_object_create          (PongObject *, int, int, int, int, Size, Size, CanvasColor);

extern Bool         pong_object_intersect       (const PongObject *, int, int, int, int);
extern Bool         pong_object_collision       (const PongObject *, const PongObject *);
extern void         pong_object_bounce          (PongObject *);
extern void         pong_object_move            (PongObject *);

extern Bool         pong_arena_create           (PongArena *, int, int, int, int, int, int, Size);
extern void         pong_arena_destroy          (PongArena *);

extern void         pong_arena_draw             (PongArena *);

extern Bool         pong_arena_intersect        (PongArena *, int, int, int, int);
extern PongObject * pong_arena_collision        (PongArena *, const PongObject *);
extern void         pong_arena_motion           (PongArena *);

extern void         pong_arena_play             (PongArena *);

#endif /* __DEBUGGER_PONG_H__ */

