#ifndef __I386_MANAGE_H__
#define __I386_MANAGE_H__

#include <utility/types.h>
#include <utility/inline.h>

typedef struct
{
	uint16	limit_0;
	uint16	base_0;
        uint8	base_1;
	uint8	attributes;
	uint8	limit_1;
	uint8	base_2;
} i386SegmentDescriptor;

typedef struct
{
	uint16	offset_0;
	uint16	selector;
	uint8	parameters;
	uint8	attributes;
	uint16	offset_1;
} i386GateDescriptor;

typedef enum
{
	I386_DESCRIPTOR_ATTRIBUTE_ACCESSED           = 0x01,
	I386_DESCRIPTOR_ATTRIBUTE_CODE               = 0x08,
	I386_DESCRIPTOR_ATTRIBUTE_NON_SYSTEM         = 0x10,
	I386_DESCRIPTOR_ATTRIBUTE_DPL                = 0x60,
	I386_DESCRIPTOR_ATTRIBUTE_PRESENT            = 0x80,
	
	I386_DESCRIPTOR_ATTRIBUTE_DPL_0              = 0x00,
	I386_DESCRIPTOR_ATTRIBUTE_DPL_1              = 0x20,
	I386_DESCRIPTOR_ATTRIBUTE_DPL_2              = 0x40,
	I386_DESCRIPTOR_ATTRIBUTE_DPL_3              = 0x60,
	
	I386_DESCRIPTOR_ATTRIBUTE_WRITE_ENABLE       = 0x02,
	I386_DESCRIPTOR_ATTRIBUTE_EXPAND_DOWN        = 0x04,

	I386_DESCRIPTOR_ATTRIBUTE_READ_ENABLE        = 0x02,
	I386_DESCRIPTOR_ATTRIBUTE_CONFORMING         = 0x04,

        I386_DESCRIPTOR_ATTRIBUTE_PAGE_GRANULARITY   = 0x80,
        I386_DESCRIPTOR_ATTRIBUTE_32_BIT             = 0x40,

	I386_DESCRIPTOR_ATTRIBUTE_TYPE               = 0x0F,

	I386_DESCRIPTOR_ATTRIBUTE_TYPE_286_TSS       = 0x01,
	I386_DESCRIPTOR_ATTRIBUTE_TYPE_286_TSS_BUSY  = 0x03,
	I386_DESCRIPTOR_ATTRIBUTE_TYPE_286_CALL_GATE = 0x04,
	I386_DESCRIPTOR_ATTRIBUTE_TYPE_286_INT_GATE  = 0x06,
	I386_DESCRIPTOR_ATTRIBUTE_TYPE_286_TRAP_GATE = 0x07,

	I386_DESCRIPTOR_ATTRIBUTE_TYPE_LDT           = 0x02,
	I386_DESCRIPTOR_ATTRIBUTE_TYPE_TASK_GATE     = 0x05,
	I386_DESCRIPTOR_ATTRIBUTE_TYPE_TSS           = 0x09,
	I386_DESCRIPTOR_ATTRIBUTE_TYPE_TSS_BUSY      = 0x0B,
	I386_DESCRIPTOR_ATTRIBUTE_TYPE_CALL_GATE     = 0x0C,
	I386_DESCRIPTOR_ATTRIBUTE_TYPE_INT_GATE      = 0x0E,
	I386_DESCRIPTOR_ATTRIBUTE_TYPE_TRAP_GATE     = 0x0F
} i386DescriptorAttribute;

typedef struct
{
	uint32	backLink;
	uint32	ESP_0;
	uint32	SS_0;
	uint32	ESP_1;
	uint32	SS_1;
	uint32	ESP_2;
	uint32	SS_2;
	uint32	CR3;
	uint32	EIP;
	uint32	EFLAGS;
	uint32	EAX;
	uint32	ECX;
	uint32	EDX;
	uint32	EBX;
	uint32	ESP;
	uint32	EBP;
	uint32	ESI;
	uint32	EDI;
	uint32	ES;
	uint32	CS;
	uint32	SS;
	uint32	DS;
	uint32	FS;
	uint32	GS;
	uint32  LDTR;
        uint16  debugTrap;
	uint16	ioBitmap;
} i386TaskStateSegment;

typedef enum
{
  I386_CR0_PROTECTION_ENABLE   = (1 << 0),
  I386_CR0_MONITOR_COPROCESSOR = (1 << 1),
  I386_CR0_EMULATION           = (1 << 2),
  I386_CR0_TASK_SWITCHED       = (1 << 3),
  I386_CR0_EXTENSION_TYPE      = (1 << 4),
  I386_CR0_NUMERIC_ERROR       = (1 << 5),
  I386_CR0_WRITE_PROTECT       = (1 << 16),
  I386_CR0_ALIGNMENT_MASK      = (1 << 18),
  I386_CR0_NOT_WRITE_THROUGH   = (1 << 29),
  I386_CR0_CACHE_DISABLE       = (1 << 30),
  I386_CR0_PAGING              = (1 << 31)
} i386CR0Flag;

typedef enum
{
  I386_CR3_WRITE_THROUGH = (1 << 3),
  I386_CR3_CACHE_DISABLE = (1 << 4),

  I386_CR3_DEFAULT       = 0
} i386CR3Flag;

typedef enum
{
  I386_CR4_VIRTUAL_8086_MODE              = (1 << 0),
  I386_CR4_VIRTUAL_INTERRUPTS             = (1 << 1),
  I386_CR4_TIME_STAMP_DISABLE             = (1 << 2),
  I386_CR4_DEBUG_EXTENSION                = (1 << 3),
  I386_CR4_PAGE_SIZE_EXTENSION            = (1 << 4),
  I386_CR4_PHYSICAL_ADDRESS_EXTENSION     = (1 << 5),
  I386_CR4_MACHINE_CHECK_ENABLE           = (1 << 6),
  I386_CR4_PAGE_GLOBAL_ENABLE             = (1 << 7),
  I386_CR4_PERFORMANCE_MONITORING_COUNTER = (1 << 8),
  I386_CR4_OS_FXSAVE_FXRSTOR              = (1 << 9),
  I386_CR4_OS_UNMASKED_EXCEPTION_SUPPORT  = (1 << 10)
} i386CR4Flag;

typedef enum
{
  I386_LDTR_DEFAULT = 0
} i386LDTRAttributes;

typedef enum
{
  I386_SELECTOR_TSS            = 0x0018,
  I386_SELECTOR_SYSTEM_CODE    = 0x0020,
  I386_SELECTOR_SYSTEM_DATA    = 0x0028,
  I386_SELECTOR_USER_CODE      = 0x0030,
  I386_SELECTOR_USER_DATA      = 0x0038
} i386Selector;

typedef enum
{
  I386_GDT_OFFSET             = 0x2000,
  I386_GDT_LIMIT              = 256 * 8,

  I386_IDT_OFFSET             = 0x2800,
  I386_IDT_LIMIT              = 256 * 8,

  I386_TSS_OFFSET             = 0x3000,
  I386_TSS_LIMIT              = 0x68,

  I386_STACK_OFFSET           = 0x0000,
  I386_STACK_LIMIT            = 8192,

  I386_STACK_INTERRUPT_OFFSET = 0x1000,
  I386_STACK_INTERRUPT_LIMIT  = 4096,

  I386_STACK_EXCEPTION_OFFSET = 0x0000,
  I386_STACK_EXCEPTION_LIMIT  = 4096,

  I386_MEMORY_OFFSET          = 0x4000,
  I386_MEMORY_LIMIT           = (~0 << 12) - I386_MEMORY_OFFSET + (1 << 12)
} i386Offset;

extern void 	i386_set_IDTR 			(i386GateDescriptor *, uint32);
extern void 	i386_get_IDTR 			(i386GateDescriptor **, uint32 *);

extern void 	i386_set_GDTR 			(i386GateDescriptor *, uint32);
extern void 	i386_get_GDTR 			(i386GateDescriptor **, uint32 *);

extern void     i386_interrupt_wait             (void);

#ifdef __GNUC__
#include <utility/assembly.h>

INLINE void	i386_set_LDTR			(uint16 value)
{ ASSEMBLY ("lldt %0" : : "r" (value)); }

INLINE uint16	i386_get_LDTR			(void)
{
  uint16 value;

  ASSEMBLY ("sldt %0" : "=r" (value) :);

  return value;
}

INLINE void 	i386_set_TR			(uint16 value)
{ ASSEMBLY ("ltr %0" : : "r" (value)); }
  
INLINE uint16	i386_get_TR			(void)
{
  uint16 value;

  ASSEMBLY ("str %0" : "=r" (value) :);

  return value;
}

INLINE void	i386_set_CR0			(uint32 value)
{ ASSEMBLY ("mov %0, %%cr0" : : "r" (value)); }

INLINE uint32	i386_get_CR0			(void)
{
   uint32 value;

   ASSEMBLY ("mov %%cr0, %0" : "=r" (value) :);

   return value;
}

INLINE void     i386_set_CR2                    (uint32 value)
{ ASSEMBLY ("mov %0, %%cr2" : : "r" (value)); }

INLINE uint32   i386_get_CR2                    (void)
{
   uint32 value;

   ASSEMBLY ("mov %%cr2, %0" : "=r" (value) :);

   return value;
}

INLINE void     i386_set_CR3                    (uint32 value)
{ ASSEMBLY ("mov %0, %%cr3" : : "r" (value)); }

INLINE uint32   i386_get_CR3                    (void)
{
   uint32 value;

   ASSEMBLY ("mov %%cr3, %0" : "=r" (value) :);

   return value;
}

INLINE void     i386_set_CR4                    (uint32 value)
{ ASSEMBLY ("mov %0, %%cr4" : : "r" (value)); }

INLINE uint32   i386_get_CR4                    (void)
{
   uint32 value;

   ASSEMBLY ("mov %%cr4, %0" : "=r" (value) :);

   return value;
}

INLINE void     i386_invalidate_page            (uint32 address)
{ ASSEMBLY ("invlpg (%0)" : : "r" (address)); }

INLINE Bool	i386_atomic_test_and_set	(Bool * target)
{
   uint8 result;

   ASSEMBLY ("lock; bts $0, %1\n"
	     "setc %0"
	     : "=r" (result), "=m" (* target));

   return result;
}

INLINE void 	i386_interrupts_enable 		(void)
{ ASSEMBLY ("sti"); }

INLINE void 	i386_interrupts_disable 	(void)
{ ASSEMBLY ("cli"); }

INLINE void     i386_clear_task_switched        (void)
{ ASSEMBLY ("clts"); }

#else /* ! __GNUC__ */

extern void     i386_set_LDTR                   (uint16);
extern uint16   i386_get_LDTR                   (void);
extern void     i386_set_TR                     (uint16);
extern uint16   i386_get_TR                     (void);
extern void     i386_set_CR0                    (uint32);
extern uint32   i386_get_CR0                    (void);
extern void     i386_set_CR2                    (uint32);
extern uint32   i386_get_CR2                    (void);
extern void     i386_set_CR3                    (uint32);
extern uint32   i386_get_CR3                    (void);
extern void     i386_set_CR4                    (uint32);
extern uint32   i386_get_CR4                    (void);

extern void     i386_invalidate_page            (uint32);
extern Bool     i386_atomic_test_and_set        (Bool *);
extern void     i386_interrupts_enable          (void);
extern void     i386_interrupts_disable         (void);
extern void     i386_clear_task_switched        (void);

#endif /* __GNUC__ */

#endif /* __I386_MANAGE_H__ */

