#include <console/console.h>
#include <ide/drive.h>
#include <ide/controller.h>

char *
ide_drive_identification_descramble (char * string, Size length)
{
        char * currentChar = string;
       
        while (length > 0)
        {
            char tmp = currentChar [0];

            currentChar [0] = currentChar [1];
            currentChar [1] = tmp;

            currentChar += 2;
            length      -= 2;
        }

        while (currentChar > string)
        {
            -- currentChar;
            if (* currentChar > ' ')
              break;

            * currentChar = '\0';
        }

        return string;
}

IDEDrive *
ide_drive_detect (IDEController * controller, IDEDriveNumber number)
{
        IDEDrive * drive = & controller -> drives [number];
        IDEError error;
        Size size;

        drive -> controller = controller;
        drive -> number     = number;
        drive -> has32BitIO = FALSE;
                
        ide_drive_recalibrate (drive);

        error = ide_drive_identify (drive);
        
        if (error != IDE_ERROR_NONE)
          return NULL;
        
        ide_drive_identification_descramble (drive -> identification.modelNumber, 40);
        ide_drive_identification_descramble (drive -> identification.serialNumber, 20);
        ide_drive_identification_descramble (drive -> identification.firmwareRevision, 8);
        
        drive -> isDetected      = TRUE;
        drive -> cylinders       = drive -> identification.logicalCylinders;
        drive -> heads           = drive -> identification.logicalHeads;
        drive -> sectorsPerTrack = drive -> identification.logicalSectorsPerTrack;
        drive -> bufferSize      = drive -> identification.bufferSize * 512;
        drive -> has32BitIO      = drive -> identification.doubleWordTransfer;

        size = drive -> cylinders * drive -> heads * drive -> sectorsPerTrack * 512 / 1024;
       
        console_print_format (NULL,
                              "IDE Controller %u, Drive %u detected:\n"
                              "%u %s, Cylinders: %u, Heads: %u, Sectors per track: %u\n"
                              "Model: %.40s\n"
                              "Serial Number: %.20s\n" 
                              "Firmware Revision: %.8s\n",
                              controller -> number,
                              number,
                              (size > 4096) ? size / 1024 : size,
                              (size > 4096) ? "MB" : "KB",
                              drive -> cylinders,
                              drive -> heads,
                              drive -> sectorsPerTrack,
                              drive -> identification.modelNumber,
                              drive -> identification.serialNumber,
                              drive -> identification.firmwareRevision);
       
       return drive;
}

IDEDrive *
ide_drive_allocate (IDEController * controller, IDEDriveNumber number)
{
        IDEDrive * drive = & controller -> drives [number];

        if (drive -> isAllocated == TRUE ||
            drive -> isDetected == FALSE)
          return NULL;

        drive -> isAllocated = TRUE;

        return drive;
}

void
ide_drive_free (IDEDrive * drive)
{
        drive -> isAllocated = FALSE;
}

