#ifndef __IDE_IDE_H__
#define __IDE_IDE_H__

#include <utility/types.h>

typedef enum
{
        IDE_COMMAND_REGISTER_DATA             = 0,
        IDE_COMMAND_REGISTER_ERROR            = 1,
        IDE_COMMAND_REGISTER_SECTOR_COUNT     = 2,
        IDE_COMMAND_REGISTER_SECTOR_NUMBER    = 3,
        IDE_COMMAND_REGISTER_CYLINDER_LOW     = 4,
        IDE_COMMAND_REGISTER_CYLINDER_HIGH    = 5,
        IDE_COMMAND_REGISTER_DRIVE_HEAD       = 6,
        IDE_COMMAND_REGISTER_STATUS           = 7,
        IDE_COMMAND_REGISTER_COMMAND          = 7
} IDECommandRegister;

typedef enum
{
        IDE_CONTROL_REGISTER_ALTERNATE_STATUS = 6,
        IDE_CONTROL_REGISTER_DEVICE_CONTROL   = 6,
        IDE_CONTROL_REGISTER_DRIVE_ADDRESS    = 7
} IDEControlRegister;

typedef enum
{
        IDE_ERROR_NONE                     = 0,
        IDE_ERROR_ADDRESS_MARK_NOT_FOUND   = (1 << 0),
        IDE_ERROR_TRACK_0_NOT_FOUND        = (1 << 1),
        IDE_ERROR_COMMAND_ABORTED          = (1 << 2),
        IDE_ERROR_SECTOR_NOT_FOUND         = (1 << 4),
        IDE_ERROR_UNCORRECTABLE_DATA_ERROR = (1 << 6),
        IDE_ERROR_BAD_BLOCK                = (1 << 7)
} IDEError;

typedef enum
{
        IDE_STATUS_COMMAND_ERROR           = (1 << 0),
        IDE_STATUS_INDEX_MARK_DETECTED     = (1 << 1),
        IDE_STATUS_CORRECTABLE_DATA_ERROR  = (1 << 2),
        IDE_STATUS_DATA_REQUEST_READY      = (1 << 3),
        IDE_STATUS_DRIVE_SEEK_COMPLETE     = (1 << 4),
        IDE_STATUS_DRIVE_WRITE_FAULT       = (1 << 5),
        IDE_STATUS_DRIVE_READY             = (1 << 6),
        IDE_STATUS_DRIVE_BUSY              = (1 << 7)
} IDEStatus;

typedef enum
{
        IDE_COMMAND_RESET_DRIVE                  = 0x08,

        IDE_COMMAND_RECALIBRATE_DRIVE            = 0x10,

        IDE_COMMAND_READ_SECTORS_WITH_RETRY      = 0x20,
        IDE_COMMAND_READ_SECTORS_WITHOUT_RETRY   = 0x21,
        IDE_COMMAND_READ_LONG_WITH_RETRY         = 0x22,
        IDE_COMMAND_READ_LONG_WITHOUT_RETRY      = 0x23,

        IDE_COMMAND_WRITE_SECTORS_WITH_RETRY     = 0x30,
        IDE_COMMAND_WRITE_SECTORS_WITHOUT_RETRY  = 0x31,
        IDE_COMMAND_WRITE_LONG_WITH_RETRY        = 0x32,
        IDE_COMMAND_WRITE_LONG_WITHOUT_RETRY     = 0x32,

        IDE_COMMAND_VERIFY_SECTORS_WITH_RETRY    = 0x40,
        IDE_COMMAND_VERIFY_SECTORS_WITHOUT_RETRY = 0x41,

        IDE_COMMAND_FORMAT_TRACK                 = 0x50,

        IDE_COMMAND_SEEK                         = 0x70,

        IDE_COMMAND_EXECUTE_DIAGNOSTIS           = 0x90,
        IDE_COMMAND_INITIALIZE_DRIVE_PARAMETERS  = 0x91,

        IDE_COMMAND_READ_BUFFER                  = 0xE4,

        IDE_COMMAND_WRITE_BUFFER                 = 0xE8,

        IDE_COMMAND_IDENTIFY_DRIVE               = 0xEC,
        
        IDE_COMMAND_SET_DRIVE_FEATURES           = 0xEF
} IDECommand;
       
typedef struct
{
        uint16  configurationInformation;
        uint16  logicalCylinders;
        uint16  reserved_0;
        uint16  logicalHeads;
        uint16  unformattedBytesPerTrack;
        uint16  unformattedBytesPerSector;
        uint16  logicalSectorsPerTrack;
        uint8   interSectorGapBytes;
        uint8   interSectorGapAfterIndexAndBeforeSplice;
        uint8   phaseLockOscillatorBytes;
        uint8   reserved_1;
        uint16  vendorUniqueStatusWords;
        char    serialNumber [20];
        uint16  controllerType;
        uint16  bufferSize;
        uint16  eccBytes;
        char    firmwareRevision [8];
        char    modelNumber [40];
        uint16  readWriteMultiplesImplemented;
        uint16  doubleWordTransfer;
        uint16  reserved_2;
        uint16  reserved_3;
        uint16  minimumPIODataTransferCycleTime;
        uint16  minimumDMADataTransferCycleTime;
} IDEDriveIdentification;

#endif /* __IDE_IDE_H__ */


