%include "utility/assembly.i"
%include "i386/manage.i"
%include "interrupt/irq.i"

section .text read nowrite exec contents

%macro EXCEPTION_STUB 1-2
.exceptionStub%1:
%if (%0 > 1)
        push    byte %2
%endif
        push    dword %1
        jmp     exceptionDispatch

        align 16
%endmacro


%macro INTERRUPT_STUB 1
.interruptStub%1:
        push    byte 0
        push    dword %1
        jmp     interruptDispatch

        align 16
%endmacro


DATA irq_stubs
%assign vector 0
%rep (7 - 0 + 1)
        EXCEPTION_STUB vector, 0
%assign vector (vector + 1)
%endrep
        EXCEPTION_STUB 8
        EXCEPTION_STUB 9, 0
%assign vector 10
%rep (14 - 10 + 1)
        EXCEPTION_STUB vector
%assign vector (vector + 1)
%endrep
        EXCEPTION_STUB 15, 0
        EXCEPTION_STUB 16, 0
        EXCEPTION_STUB 17
%assign vector 18
%rep (31 - 18 + 1)
        EXCEPTION_STUB vector, 0
%assign vector (vector + 1)
%endrep

%assign vector 32
%rep (IRQ_COUNT - 32)
        INTERRUPT_STUB vector
%assign vector (vector + 1)
%endrep
DATA_END


%define EXCEPTION_REGISTERS      I386_STACK_EXCEPTION_OFFSET + I386_STACK_EXCEPTION_LIMIT - IRQRegisters.size

        extern exception_dispatch

exceptionDispatch:
        mov     [EXCEPTION_REGISTERS + IRQRegisters.EAX], eax
        mov     [EXCEPTION_REGISTERS + IRQRegisters.ECX], ecx
        mov     [EXCEPTION_REGISTERS + IRQRegisters.EDX], edx
        mov     [EXCEPTION_REGISTERS + IRQRegisters.EBX], ebx
        mov     [EXCEPTION_REGISTERS + IRQRegisters.EBP], ebp
        mov     [EXCEPTION_REGISTERS + IRQRegisters.ESI], esi
        mov     [EXCEPTION_REGISTERS + IRQRegisters.EDI], edi

        pop     dword [EXCEPTION_REGISTERS + IRQRegisters.irqVector]
        pop     dword [EXCEPTION_REGISTERS + IRQRegisters.errorCode]
        pop     dword [EXCEPTION_REGISTERS + IRQRegisters.EIP]
        pop     dword [EXCEPTION_REGISTERS + IRQRegisters.CS]
        pop     dword [EXCEPTION_REGISTERS + IRQRegisters.EFLAGS]
        mov     [EXCEPTION_REGISTERS + IRQRegisters.ESP], esp

        mov     esp, EXCEPTION_REGISTERS

        call    exception_dispatch

        mov     eax, [EXCEPTION_REGISTERS + IRQRegisters.EAX]
        mov     ecx, [EXCEPTION_REGISTERS + IRQRegisters.ECX]
        mov     edx, [EXCEPTION_REGISTERS + IRQRegisters.EDX]
        mov     ebx, [EXCEPTION_REGISTERS + IRQRegisters.EBX]
        mov     ebp, [EXCEPTION_REGISTERS + IRQRegisters.EBP]
        mov     esi, [EXCEPTION_REGISTERS + IRQRegisters.ESI]
        mov     edi, [EXCEPTION_REGISTERS + IRQRegisters.EDI]

        mov     esp, [EXCEPTION_REGISTERS + IRQRegisters.ESP]
        push    dword [EXCEPTION_REGISTERS + IRQRegisters.EFLAGS]
        push    dword [EXCEPTION_REGISTERS + IRQRegisters.CS]
        push    dword [EXCEPTION_REGISTERS + IRQRegisters.EIP]

        iretd


%define INTERRUPT_REGISTERS      I386_STACK_INTERRUPT_OFFSET + I386_STACK_INTERRUPT_LIMIT - IRQRegisters.size

        extern interrupt_dispatch

interruptDispatch:
        mov     [INTERRUPT_REGISTERS + IRQRegisters.EAX], eax
        mov     [INTERRUPT_REGISTERS + IRQRegisters.ECX], ecx
        mov     [INTERRUPT_REGISTERS + IRQRegisters.EDX], edx
        mov     [INTERRUPT_REGISTERS + IRQRegisters.EBX], ebx
        mov     [INTERRUPT_REGISTERS + IRQRegisters.EBP], ebp
        mov     [INTERRUPT_REGISTERS + IRQRegisters.ESI], esi
        mov     [INTERRUPT_REGISTERS + IRQRegisters.EDI], edi

        pop     dword [INTERRUPT_REGISTERS + IRQRegisters.irqVector]
        pop     dword [INTERRUPT_REGISTERS + IRQRegisters.errorCode]
        pop     dword [INTERRUPT_REGISTERS + IRQRegisters.EIP]
        pop     dword [INTERRUPT_REGISTERS + IRQRegisters.CS]
        pop     dword [INTERRUPT_REGISTERS + IRQRegisters.EFLAGS]
        mov     [INTERRUPT_REGISTERS + IRQRegisters.ESP], esp

        mov     esp, INTERRUPT_REGISTERS

        call    interrupt_dispatch

        mov     eax, [INTERRUPT_REGISTERS + IRQRegisters.EAX]
        mov     ecx, [INTERRUPT_REGISTERS + IRQRegisters.ECX]
        mov     edx, [INTERRUPT_REGISTERS + IRQRegisters.EDX]
        mov     ebx, [INTERRUPT_REGISTERS + IRQRegisters.EBX]
        mov     ebp, [INTERRUPT_REGISTERS + IRQRegisters.EBP]
        mov     esi, [INTERRUPT_REGISTERS + IRQRegisters.ESI]
        mov     edi, [INTERRUPT_REGISTERS + IRQRegisters.EDI]

        mov     esp, [INTERRUPT_REGISTERS + IRQRegisters.ESP]
        push    dword [INTERRUPT_REGISTERS + IRQRegisters.EFLAGS]
        push    dword [INTERRUPT_REGISTERS + IRQRegisters.CS]
        push    dword [INTERRUPT_REGISTERS + IRQRegisters.EIP]

        iretd

