#ifndef __KBC_KEY_H__
#define __KBC_KEY_H__

#include <utility/types.h>

enum
{
       /* ASCII range reserved for keys where applicable. */
       /* Range above ASCII to 0x100 also reserved. */

       /* Convenient names for keys that can be sensibly
        * mapped into ASCII range.
        */
       KEY_BACKSPACE     = '\b',
       KEY_TAB           = '\t',
       KEY_ENTER         = '\n',
       KEY_ESCAPE        = 0x1B, /* '\e', */
       KEY_SPACE         = ' ',

       KEY_INSERT        = 0x100,
       KEY_DELETE        = 0x101,
       KEY_HOME          = 0x102,
       KEY_END           = 0x103,
       KEY_PAGE_UP       = 0x104,
       KEY_PAGE_DOWN     = 0x105,

       KEY_CURSOR_UP     = 0x106,
       KEY_CURSOR_DOWN   = 0x107,
       KEY_CURSOR_LEFT   = 0x108,
       KEY_CURSOR_RIGHT  = 0x109,

       KEY_SHIFT_LEFT    = 0x10A,
       KEY_SHIFT_RIGHT   = 0x10B,
       KEY_CTRL_LEFT     = 0x10C,
       KEY_CTRL_RIGHT    = 0x10D,
       KEY_ALT_LEFT      = 0x10E,
       KEY_ALT_RIGHT     = 0x10F,
       
       KEY_CAPS_LOCK     = 0x110,
       KEY_NUM_LOCK      = 0x111,
       KEY_SCROLL_LOCK   = 0x112,

       KEY_PRINT_SCREEN  = 0x113,
       KEY_SYS_REQ       = 0x114,
       KEY_PAUSE         = 0x115,
       KEY_BREAK         = 0x116,

       KEY_F1            = 0x117,
       KEY_F2            = 0x118,
       KEY_F3            = 0x119,
       KEY_F4            = 0x11A,
       KEY_F5            = 0x11B,
       KEY_F6            = 0x11C,
       KEY_F7            = 0x11D,
       KEY_F8            = 0x11E,
       KEY_F9            = 0x11F,
       KEY_F10           = 0x120,
       KEY_F11           = 0x121,
       KEY_F12           = 0x122,
       KEY_F13           = 0x123,
       KEY_F14           = 0x124,
       KEY_F15           = 0x125,
       KEY_F16           = 0x126,

       KEY_WINDOW_LEFT   = 0x127,
       KEY_WINDOW_RIGHT  = 0x128,
       KEY_MENU_LEFT     = 0x129,
       KEY_MENU_RIGHT    = 0x12A,

       KEY_KEYPAD_UP     = 0x12B,
       KEY_KEYPAD_DOWN   = 0x12C,
       KEY_KEYPAD_LEFT   = 0x12D,
       KEY_KEYPAD_RIGHT  = 0x12E,
       KEY_KEYPAD_CENTER = 0x12F,

       /* Signals that the modifier map should be ignored
        * for this key.
        */
       KEY_IGNORE        = 0,

       /* Signals that the key is invalid. */
       KEY_ERROR         = 0xFFFF
};

typedef uint16 Key;

#endif /* __KBC_KEY_H__ */

