#include <utility/string.h>
#include <memory/memory.h>
#include <kbc/key_translation.h>

static List keyTranslations;

Bool
__key_translation_enter__ (void)
{
        list_clear (& keyTranslations);

        return TRUE;
}

void
__key_translation_exit__ (void)
{
        while (list_empty (& keyTranslations) == FALSE)
          key_translation_destroy ((KeyTranslation *) list_front (& keyTranslations));
}

Bool
key_translation_create (KeyTranslation ** result,
                        const char * name,
                        KeyModifierMap * modifierMaps,
                        Size modifierMapCount,
                        KeyMap * maps,
                        Size mapCount)
{
        KeyTranslation * translation;
 
        if (modifierMapCount < 1 ||
            mapCount < 1)
          return FALSE;

        translation = (KeyTranslation *) memory_allocate (sizeof (KeyTranslation));

        if (translation == NULL)
          return FALSE;

        translation -> name             = name;
        translation -> modifierMaps     = modifierMaps;
        translation -> modifierMapCount = modifierMapCount;
        translation -> maps             = maps;
        translation -> mapCount         = mapCount;

        list_insert_front (& keyTranslations, translation);

        if (result != NULL)
          * result = translation;

        return TRUE;
}

void
key_translation_destroy (KeyTranslation * translation)
{
        list_remove (& translation -> translationList);

        memory_free (translation);
}

KeyTranslation *
key_translation_find (const char * name)
{
        ListIterator position;

        for (position = list_begin (& keyTranslations);
             position != list_end (& keyTranslations);
             position = list_next (position))
        {
           KeyTranslation * translation = (KeyTranslation *) position;

           if (string_equal (translation -> name, name) == TRUE)
             return translation;
        }

        return NULL;
}

KeyTranslation *
key_translation_default (void)
{
        if (list_empty (& keyTranslations) == TRUE)
          return NULL;

        return (KeyTranslation *) list_front (& keyTranslations);
}

Key
key_translation_translate (const KeyTranslation * translation, KeyModifiers modifiers, uint8 scancode)
{
        KeyModifierMap * currentModifierMap;

        for (currentModifierMap = translation -> modifierMaps;
             currentModifierMap < & translation -> modifierMaps [translation -> modifierMapCount];
             ++ currentModifierMap)
        {
            if ((currentModifierMap -> modifiers & modifiers) == currentModifierMap -> modifiers)
            {
               Key key = translation -> maps [currentModifierMap -> map].keys [scancode];

               if (key != KEY_IGNORE)
                 return key;
            }
        }

        return KEY_ERROR;
}

