#ifndef __KBC_KEY_TRANSLATION_H__
#define __KBC_KEY_TRANSLATION_H__

#include <utility/types.h>
#include <utility/list.h>
#include <kbc/key.h>

enum
{
        KEY_MAP_SIZE = 128
};

typedef enum
{
        KEY_MODIFIER_NONE    = 0,
        KEY_MODIFIER_DEFAULT = 0,

        KEY_MODIFIER_EXTENDED    = (1 << 0),
        KEY_MODIFIER_SHIFT       = (1 << 1),
        KEY_MODIFIER_CTRL        = (1 << 2),
        KEY_MODIFIER_ALT         = (1 << 3),
        KEY_MODIFIER_SCROLL_LOCK = (1 << 4),
        KEY_MODIFIER_NUM_LOCK    = (1 << 5),
        KEY_MODIFIER_CAPS_LOCK   = (1 << 6)
} KeyModifier;

typedef FlagSet KeyModifiers;

typedef struct
{
        KeyModifiers modifiers;
        Size         map;
} KeyModifierMap;

typedef struct
{
        Key keys [KEY_MAP_SIZE];
} KeyMap;

typedef struct
{
        ListNode         translationList;
        const char *     name;
        KeyModifierMap * modifierMaps;
        Size             modifierMapCount;
        KeyMap *         maps;
        Size             mapCount;
} KeyTranslation;

extern Bool     key_translation_create       (KeyTranslation **, const char *, KeyModifierMap *, Size, KeyMap *, Size);
extern void     key_translation_destroy      (KeyTranslation *);

extern KeyTranslation * key_translation_find    (const char *);
extern KeyTranslation * key_translation_default (void);

extern Key      key_translation_translate    (const KeyTranslation *, KeyModifiers, uint8);

#endif /* __KBC_KEY_TRANSLATION_H__ */

