#include <kbc/key_translation.h>

static KeyMap usEnglish101Maps [5] =
{
  { /* Default map */ 
    { 
      KEY_ERROR, KEY_ESCAPE, '1', '2', '3', '4', '5', '6', 
      '7', '8', '9', '0', '-', '=', KEY_BACKSPACE, KEY_TAB, 
      'q', 'w', 'e', 'r', 't', 'y', 'u', 'i', 
      'o', 'p', '[', ']', KEY_ENTER, KEY_CTRL_LEFT, 'a', 's',
      'd', 'f', 'g', 'h', 'j', 'k', 'l', ';', 
      '\'', '`', KEY_SHIFT_LEFT, '\\', 'z', 'x', 'c', 'v',
      'b', 'n', 'm', ',', '.', '/', KEY_SHIFT_RIGHT, KEY_PRINT_SCREEN,
      KEY_ALT_LEFT, KEY_SPACE, KEY_CAPS_LOCK, KEY_F1, KEY_F2, KEY_F3, KEY_F4, KEY_F5,
      KEY_F6, KEY_F7, KEY_F8, KEY_F9, KEY_F10, KEY_NUM_LOCK, KEY_SCROLL_LOCK, KEY_HOME,
      KEY_KEYPAD_UP, KEY_PAGE_UP, '-', KEY_KEYPAD_LEFT, KEY_KEYPAD_CENTER, KEY_KEYPAD_RIGHT, '+', KEY_END,
      KEY_KEYPAD_DOWN, KEY_PAGE_DOWN, KEY_INSERT, KEY_DELETE
    }
  },
  { /* Extended map */
    {
      0, 0, 0, 0, 0, 0, 0, 0,
      0, 0, 0, 0, 0, 0, 0, 0,
      0, 0, 0, 0, 0, 0, 0, 0,
      0, 0, 0, 0, 0, 0, 0, 0,
      0, 0, 0, 0, KEY_ENTER, KEY_CTRL_RIGHT, 0, 0,
      0, 0, 0, 0, 0, 0, 0, 0,
      0, 0, 0, 0, 0, 0, 0, 0,
      KEY_ALT_RIGHT, 0, 0, 0, 0, 0, 0, 0,
      0, 0, 0, 0, 0, 0, 0, KEY_HOME,
      KEY_CURSOR_UP, KEY_PAGE_UP, 0, KEY_CURSOR_LEFT, 0, KEY_CURSOR_RIGHT, 0, KEY_END,
      KEY_CURSOR_DOWN, KEY_PAGE_DOWN, KEY_INSERT, KEY_DELETE, 0, 0, 0, 0,
      0, 0, 0, 0, KEY_WINDOW_LEFT, KEY_WINDOW_RIGHT, KEY_MENU_RIGHT, 0
    }
  },
  { /* Shift map */
    {  
      0, 0, '!', '@', '#', '$', '%', '^', 
      '&', '*', '(', ')', '_', '+', 0, 0,
      'Q', 'W', 'E', 'R', 'T', 'Y', 'U', 'I', 
      'O', 'P', '{', '}', 0, 0, 'A', 'S',
      'D', 'F', 'G', 'H', 'J', 'K', 'L', ':',
      '"', '~', 0, '|', 'Z', 'X', 'C', 'V',
      'B', 'N', 'M', '<', '>', '?', 0, 0,
      0, 0, 0, 0, 0, 0, 0, 0,
      0, 0, 0, 0, 0, 0, 0, '7',
      '8', '9', 0, '4', '5', '6', 0, '1',
      '2', '3', '0', '.'
    }
  },
  { /* Caps lock map */
    {
      0, 0, 0, 0, 0, 0, 0, 0,
      0, 0, 0, 0, 0, 0, 0, 0,
      'Q', 'W', 'E', 'R', 'T', 'Y', 'U', 'I',
      'O', 'P', 0, 0, 0, 0, 'A', 'S',
      'D', 'F', 'G', 'H', 'J', 'K', 'L', 0,
      0, 0, 0, 0, 'Z', 'X', 'C', 'V',
      'B', 'N', 'M', 0, 0, 0, 0, 0,
      0, 0, 0, 0, 0, 0, 0, 0,
      0, 0, 0, 0, 0, 0, 0, 0,
      0, 0, 0, 0, 0, 0, 0, 0,
      0, 0, 0, 0
    }
  },
  { /* Num lock map */
    {  
      0, 0, 0, 0, 0, 0, 0, 0,
      0, 0, 0, 0, 0, 0, 0, 0,
      0, 0, 0, 0, 0, 0, 0, 0,
      0, 0, 0, 0, 0, 0, 0, 0,
      0, 0, 0, 0, 0, 0, 0, 0,
      0, 0, 0, 0, 0, 0, 0, 0,
      0, 0, 0, 0, 0, 0, 0, 0,
      0, 0, 0, 0, 0, 0, 0, 0,
      0, 0, 0, 0, 0, 0, 0, '7',
      '8', '9', 0, '4', '5', '6', 0, '1',
      '2', '3', '0', '.'
    }
  }
}; 

static KeyModifierMap usEnglish101ModifierMaps [5] =
{
  { KEY_MODIFIER_EXTENDED, 1 },
  { KEY_MODIFIER_SHIFT, 2 },
  { KEY_MODIFIER_CAPS_LOCK, 3 },
  { KEY_MODIFIER_NUM_LOCK, 4 },
  { KEY_MODIFIER_NONE, 0 }
};

Bool
__us_english_101_enter__ (void)
{
   if (key_translation_create (NULL, 
                               "us-english-101",
                               usEnglish101ModifierMaps,
                               sizeof (usEnglish101ModifierMaps) / sizeof (KeyModifierMap),
                               usEnglish101Maps,
                               sizeof (usEnglish101Maps) / sizeof (KeyMap)) == FALSE)
     return FALSE;

   return TRUE;
}

