#include <scheduler/thread.h>
#include <console/console.h>
#include <kbc/keyboard.h>
#include <libc/stdio.h>

static const char * keyNames [] =
{
    "Insert",
    "Delete",
    "Home",
    "End",
    "Page up",
    "Page down",
    "Cursor up",
    "Cursor down",
    "Cursor left",
    "Cursor right",
    "Shift left",
    "Shift right",
    "Ctrl left",
    "Ctrl right",
    "Alt left",
    "Alt right",
    "Caps lock",
    "Num lock",
    "Scroll lock",
    "Print screen",
    "Sys req",
    "Pause",
    "Break",
    "F1",
    "F2",
    "F3",
    "F4",
    "F5",
    "F6",
    "F7",
    "F8",
    "F9",
    "F10",
    "F11",
    "F12",
    "F13",
    "F14",
    "F15",
    "F16",
    "Window left",
    "Window right",
    "Menu left",
    "Menu right",
    "Keypad up",
    "Keypad down",
    "Keypad left",
    "Keypad right",
    "Keypad center"
};

void
keyboardTest (void * data)
{
    char message [1024];
    Keyboard keyboard;
    KeyEvent event;

    console_print (NULL, "Enter a message: ");

    if (fgets (message, sizeof (message), stdin) > 0)
      console_print_format (NULL, "You entered: %s\n", message);
    else
      console_print (NULL, "kbd_test: fgets () failed!\n");

    if (keyboard_create (& keyboard, NULL, NULL, NULL, NULL, NULL, NULL, NULL) == FALSE)
    {
       console_print (NULL, "kbd_test: Failed to create keyboard!\n");

       return;
    }

    keyboard_activate (& keyboard);
    
    while (keyboard_read_events (keyboard_current (), & event, 1) == 1)
    {
        if (event.key > ' ' && event.key <= '~')
          console_print_format (NULL,
                                "kbd_test: %c (%u/%x) %s\n",
                                 event.key,
                                 event.key,
                                 event.key,
                                 event.state == KEY_STATE_PRESSED ? "pressed" : "released");
        else
          console_print_format (NULL,
                                "kbd_test: %s (%u/%x) %s\n",
                                event.key == KEY_ERROR ? "Error" :
                                   event.key == KEY_SPACE ? "Space" :
                                   event.key == KEY_BACKSPACE ? "Backspace" :
                                   event.key == KEY_ENTER ? "Enter" :
                                   event.key == KEY_TAB ? "Tab" : keyNames [event.key - 0x100],
                                event.key,
                                event.key,
                                event.state == KEY_STATE_PRESSED ? "pressed" : "released");
    }

    console_print (NULL, "kbd_test: Failed to read keyboard!\n");

    keyboard_destroy (& keyboard);
}

Bool
__kbd_test_enter__ (void)
{
    return thread_create (NULL,
                          THREAD_PRIORITY_MAXIMUM,
                          THREAD_PRIORITY_MINIMUM,
                          keyboardTest,
                          NULL,
                          NULL,
                          0);
}


