#ifndef __LIBC_STDIO_H__
#define __LIBC_STDIO_H__

#include <libc/stdlib.h>
#include <libc/stdarg.h>

typedef void	FILE;

enum
{
   EOF = -1
};

extern FILE *	stdin;
extern FILE *	stdout;
extern FILE *	stderr;

extern FILE *	fopen		(const char *, const char *);
extern int	fclose		(FILE *);
extern size_t	fread		(void *, size_t, size_t, FILE *);
extern size_t	fwrite		(const void *, size_t, size_t, FILE *);
extern int	feof		(FILE *);
extern int	fflush		(FILE *);

extern int	fputc		(int, FILE *);
extern int      fputs           (const char *, FILE *);
extern int	puts		(const char *);
extern int	printf		(const char *, ...);
extern int	fprintf		(FILE *, const char *, ...);
extern int	sprintf		(char *, const char *, ...);
extern int	snprintf	(char *, size_t, const char *, ...);
extern int	vprintf		(const char *, va_list);
extern int	vfprintf	(FILE *, const char *, va_list);
extern int	vsprintf	(char *, const char *, va_list);
extern int	vsnprintf	(char *, size_t, const char *, va_list);

extern int      fgetc           (FILE *);
extern char *   fgets           (char *, int, FILE *);
extern char *   gets            (char *);
extern int      ungetc          (int, FILE *);

extern int	sscanf		(const char *, const char *, ...);

#define putc(c, stream) \
        (fputc ((c), (stream)))

#define putchar(c) \
        (putc ((c), stdout)

#define getc(stream) \
        (fgetc ((stream)))

#define getchar() \
        (getc (stdin))

#endif /* __LIBC_STDIO_H__ */

