#include <scheduler/semaphore.h>
#include <timer/pit.h>
#include <timer/timer.h>
#include <libc/stdlib.h>
#include <libc/sys/times.h>
#include <libc/time.h>

time_t
time (time_t * timeResult)
{
   time_t seconds = pit_count () / TIMER_HZ;

   if (timeResult != NULL)
     * timeResult = seconds;
	
   return seconds;
}

clock_t
clock (void)
{
   return pit_count () * CLOCKS_PER_SEC / TIMER_HZ;
}

clock_t
times (struct tms * tms)
{
   if (tms == NULL)
     return clock ();

   tms -> tms_utime  = clock ();
   tms -> tms_stime  = 0;
   tms -> tms_cutime = 0;
   tms -> tms_cstime = 0;

   return tms -> tms_utime;
}

static void
nanosleep_handler (Timer * timer, void * data)
{
    semaphore_up ((Semaphore *) data);
}

int
nanosleep (const struct timespec * request, struct timespec * remaining)
{
   TimeOffset sleepOffset;
   Timer      sleepTimer;
   Semaphore  sleepSemaphore;
   
   if (request -> tv_sec < 0 ||
       request -> tv_nsec >= NANOSECONDS_PER_SECOND)
     return -1;

   sleepOffset.seconds     = (Seconds) request -> tv_sec;
   sleepOffset.nanoSeconds = (NanoSeconds) request -> tv_nsec;

   if (timer_create (& sleepTimer, 
                     & sleepOffset,
                     nanosleep_handler,
                     & sleepSemaphore,
                     FALSE) == FALSE)
     return -1;

   semaphore_create (& sleepSemaphore, 0);
   
   timer_start (& sleepTimer);
   semaphore_down (& sleepSemaphore);
   
   semaphore_destroy (& sleepSemaphore);
   timer_destroy (& sleepTimer);
   
   if (remaining != NULL)
   {
      remaining -> tv_sec  = 0;
      remaining -> tv_nsec = 0;
   }

   return 0;
}

char *
ctime (const time_t * time)
{
   return NULL;
}

