#ifndef __LINKER_COMPONENT_H__
#define __LINKER_COMPONENT_H__

#include <utility/list.h>
#include <utility/hash_table.h>
#include <linker/import.h>
#include <linker/export.h>
#include <linker/section.h>
#include <linker/interface.h>

struct Linker;

typedef enum
{
        COMPONENT_STATE_CONSTRUCTING = 1,
        COMPONENT_STATE_READY        = 2,
        COMPONENT_STATE_DESTRUCTING  = 3
} ComponentState;

typedef struct Component
{
        ListNode        componentList;
        struct Linker * linker;
        ComponentState  state;
        const char *    name;
        List            sections;
        List            interfaces;
        List            constructors;
        List            destructors;
        List            imports;
        HashTable       exports;
        Size            referenceCount;
} Component;

extern Bool             component_create                (Component **, const char *);
extern void             component_destroy               (Component *);

extern Export *         component_find_export           (Component *, const char *);
extern Import *         component_find_import           (Component *, const char *, const char *);
extern Interface *      component_find_interface        (Component *, const char *);
extern Section *        component_find_section          (Component *, const char *);

extern void             component_relocate_sections     (Component *);
extern Bool             component_resolve_imports       (Component *);

extern Bool             component_call_constructors     (Component *);
extern void             component_call_destructors      (Component *);

#endif /* __LINKER_COMPONENT_H__ */

