#include <utility/string.h>
#include <linker/linker.h>

static Linker   linkerDefault;
static Linker * linkerCurrent = NULL;

Bool
linker_create (Linker * linker)
{
        mutex_create (& linker -> lock);

        list_clear (& linker -> components);

        return TRUE;
}

void
linker_destroy (Linker * linker)
{
        while (list_empty (& linker -> components) == FALSE)
           component_destroy ((Component *) list_back (& linker -> components));

        mutex_destroy (& linker -> lock);
}

Linker *
linker_current (void)
{
        return linkerCurrent;
}

void
linker_activate (Linker * linker)
{
        if (linkerCurrent == NULL)
          linker_create (& linkerDefault);

        if (linker == NULL)
          linker = & linkerDefault;

        linkerCurrent = linker;
}

Export *
linker_find_export (const char * interface, const char * name)
{
        Linker * linker = linkerCurrent;
        ListIterator position;

        for (position = list_begin (& linker -> components);
             position != list_end (& linker -> components);
             position = list_next (position))
        {
            Component * component = (Component *) position;
            Export * export;
            
            if (component -> state != COMPONENT_STATE_READY)
              continue;
        
            export = component_find_export (component, name);

            if (export -> interface != NULL &&
                string_equal (export -> interface -> name, interface) == TRUE)
              return export;
        }
        
        return NULL;
}

Interface *
linker_find_interface (const char * name)
{
        Linker * linker = linkerCurrent;
        ListIterator position;

        for (position = list_begin (& linker -> components);
             position != list_end (& linker -> components);
             position = list_next (position))
        {
            Component * component = (Component *) position;
            Interface * interface;
            
            if (component -> state != COMPONENT_STATE_READY)
              continue;
            
            interface = component_find_interface (component, name);

            if (interface != NULL)
              return interface;
        }

        return NULL;
}

Component *
linker_find_component (const char * name)
{
        Linker * linker = linkerCurrent;
        ListIterator position;

        for (position = list_begin (& linker -> components);
             position != list_end (& linker -> components);
             position = list_next (position))
        {
            Component * component = (Component *) position;

            if (component -> state != COMPONENT_STATE_READY)
              continue;

            if (string_equal (component -> name, name) == TRUE)
              return component;
        }

        return NULL;
}

