#include <page/allocator.h>

Page *
page_allocator_coalesce (PageAllocator * allocator,
                         Page * page,
   	                 PageAlignment alignmentCeiling)
{
     Page * bottomHalf, 
          * topHalf;

     if (alignmentCeiling >= allocator -> freeListCount)
       alignmentCeiling = allocator -> freeListCount - 1;

     while (page -> alignment <= alignmentCeiling)
     {
	bottomHalf = page;
	if (page -> address & (1 << page -> alignment))
          bottomHalf -= (1 << page -> alignment);
	
        topHalf = bottomHalf + (1 << page -> alignment);
	
        if (bottomHalf -> owner != NULL ||
            topHalf -> owner != NULL)
          return page;

        list_remove (& bottomHalf -> pageList);
	list_remove (& topHalf -> pageList);

	list_insert_front (& allocator -> freeLists [++ bottomHalf -> alignment],
			   & bottomHalf -> pageList);

	page = bottomHalf;
     }

     return page;
}

Size
page_allocator_fill (PageAllocator * allocator,
		     PageAlignment alignmentRequested,
		     Size limit)
{
   Size result = 0;
   PageAlignment alignmentCurrent;

   for (alignmentCurrent = 0;
	alignmentCurrent < alignmentRequested;
	++ alignmentCurrent)
   {
       ListIterator pageNext = list_begin (& allocator -> freeLists [alignmentCurrent]);

       while (pageNext != list_end (& allocator -> freeLists [alignmentCurrent]))
       {
	    ListIterator pageCurrent = pageNext;

	    pageNext = list_next (pageNext);
	    
	    if (page_allocator_coalesce (allocator, (Page *) pageCurrent, alignmentRequested) -> alignment == alignmentRequested &&
	        ++ result >= limit)
	      return result;
       }
   }

   return result;
}

void
page_allocator_free (PageAllocator * allocator, 
                     Page * page)
{
   if (page -> owner == NULL)
     return;

   page -> owner = NULL;
     
   list_insert_front (& allocator -> freeLists [page -> alignment],
		      & page -> pageList);
}

