#ifndef __PCI_BIOS_H__
#define __PCI_BIOS_H__

#include <pci/pci.h>

typedef enum
{
  PCI_BIOS_ERROR_SUCCESSFUL           = 0x00,
  PCI_BIOS_ERROR_UNSUPPORTED_FUNCTION = 0x81,
  PCI_BIOS_ERROR_BAD_VENDOR_ID        = 0x83,
  PCI_BIOS_ERROR_DEVICE_NOT_FOUND     = 0x86,
  PCI_BIOS_ERROR_BAD_REGISTER_NUMBER  = 0x87,
  PCI_BIOS_ERROR_SET_FAILED           = 0x88,
  PCI_BIOS_ERROR_BUFFER_TOO_SMALL     = 0x89
} PCIBiosError;

typedef enum
{
  PCI_BIOS_FUNCTION_ID	                    = 0xB1,
  PCI_BIOS_FUNCTION_PROBE                   = 0x01,
  PCI_BIOS_FUNCTION_FIND_DEVICE             = 0x02,
  PCI_BIOS_FUNCTION_FIND_CLASS_CODE         = 0x03,
  PCI_BIOS_FUNCTION_GENERATE_SPECIAL_CYCLE  = 0x06,
  PCI_BIOS_FUNCTION_READ_CONFIG_8           = 0x08,
  PCI_BIOS_FUNCTION_READ_CONFIG_16          = 0x09,
  PCI_BIOS_FUNCTION_READ_CONFIG_32          = 0x0A,
  PCI_BIOS_FUNCTION_WRITE_CONFIG_8          = 0x0B,
  PCI_BIOS_FUNCTION_WRITE_CONFIG_16         = 0x0C,
  PCI_BIOS_FUNCTION_WRITE_CONFIG_32         = 0x0D,
  PCI_BIOS_FUNCTION_GET_IRQ_ROUTING_OPTIONS = 0x0E,
  PCI_BIOS_FUNCTION_SET_PCI_IRQ             = 0x0F
} PCIBiosFunction;

typedef uint8	PCIBiosVersion;

extern Bool		pci_bios_initialize	(void);
extern PCIBiosError	pci_bios_probe		(PCIBiosVersion *, PCIBiosVersion *, PCIBusNumber *);
extern PCIBiosError	pci_bios_find_device	(PCIDeviceID, PCIVendorID, Size, PCIBusNumber *, PCIDeviceNumber *);
extern PCIBiosError	pci_bios_find_class	(uint32, uint32, PCIBusNumber *, PCIDeviceNumber *);

extern PCIBiosError	pci_bios_read_config_8	 (uint32, uint32, uint32, uint8 *);
extern PCIBiosError	pci_bios_read_config_16	 (uint32, uint32, uint32, uint16 *);
extern PCIBiosError	pci_bios_read_config_32  (uint32, uint32, uint32, uint32 *);
extern PCIBiosError     pci_bios_write_config_8	 (uint32, uint32, uint32, uint32);
extern PCIBiosError     pci_bios_write_config_16 (uint32, uint32, uint32, uint32);
extern PCIBiosError     pci_bios_write_config_32 (uint32, uint32, uint32, uint32);

#endif /* __PCI_BIOS_H__ */

