%include "utility/assembly.i"
%include "i386/manage.i"
%include "pci/bios32.i"
%include "pci/bios.i"

section .data read write noexec contents align=4 arch=little

pciBios32EntryPoint:
	dd	0
	dw	I386_SELECTOR_SYSTEM_CODE


section .text read nowrite exec contents

	extern	console_print
	extern	console_print_format
	extern	pci_bios32_find_service_directory

FUNCTION pci_bios_initialize
	call 	pci_bios32_find_service_directory
	
	test	eax, eax
	jnz	.checkRevisionLevel

	push	dword .serviceDirectoryNotFound
	push    dword 0
        call	console_print
	add	esp, 8
	ret

.checkRevisionLevel:
	mov	ecx, eax
	mov	eax, [ecx + PCIBios32ServiceDirectory.entryPoint]
	mov	[pciBios32EntryPoint], eax

	push	eax
	push	ecx
	push	dword .serviceDirectoryFound
	push    dword 0
        call	console_print_format
	add	esp, 16

	xor	eax, eax
	mov	al, [ecx + PCIBios32ServiceDirectory.revisionLevel]
	test	al, al
	je	.queryServiceDirectory

	push	eax
	push	dword .unsupportedBios32Revision
	push    dword 0
        call	console_print_format
	add	esp, 12
	ret

.queryServiceDirectory:
	push	ebx
	mov	eax, '$' | ('P' << 8) | ('C' << 16) | ('I' << 24)
	xor	ebx, ebx
	mov	edx, pciBios32EntryPoint

	pushf
	cli
	call	far dword [edx]
	popf

	test	al, al
	jz	.setEntryPoint

	push	dword .noBiosFound
	push    dword 0
        call	console_print
	add	esp, 8

	pop	ebx
	xor	eax, eax
	ret

.setEntryPoint:
	add	ebx, edx
	mov	dword [pciBios32EntryPoint], ebx

	push	ebx
	push	dword .foundBiosEntryPoint
	push    dword 0
        call	console_print_format
	add	esp, 12

	pop	ebx
	mov	eax, 1
	ret

.unsupportedBios32Revision:
	db	"PCI BIOS32: BIOS32 revision %d is not supported.", 10, 0

.serviceDirectoryFound:
        db      "PCI BIOS32: Service directory found at %x with entry point %x.", 10, 0

.serviceDirectoryNotFound:
	db      "PCI BIOS32: Service directory not found.", 10, 0

.foundBiosEntryPoint:
        db      "PCI BIOS32: Found BIOS entry point at %x.", 10, 0

.noBiosFound:
	db	"PCI BIOS32: Could not find BIOS entry point.", 10, 0
FUNCTION_END


%macro PCI_BIOS32_CALL 1
	mov 	al, %1
	mov 	ah, PCI_BIOS_FUNCTION_ID
	push	ebp
	mov	ebp, pciBios32EntryPoint
        pushf
        cli
	call	far dword [ebp]
        popf
	pop	ebp
        movzx   eax, ah
%endmacro


FUNCTION pci_bios_probe
	push	ebx

	mov	eax, PCI_BIOS_FUNCTION_PROBE | (PCI_BIOS_FUNCTION_ID << 8)
	mov	ebx, pciBios32EntryPoint
	
	pushf
	cli
	call	far dword [ebx]
	jc	.returnError

	cmp	edx, 'P' | ('C' << 8) | ('I' << 16) | (' ' << 24)
	je	.returnError

	mov	eax, PCI_BIOS_ERROR_UNSUPPORTED_FUNCTION << 8

.returnError:
	popf

        movzx   eax, ah

	mov	edx, [esp + 4 * 2 + 4 * 2]
	mov	byte [edx], cl
	mov	ecx, [esp + 4 * 2 + 4 * 0]
	mov	edx, [esp + 4 * 2 + 4 * 1]
	mov	byte [ecx], bh
	mov	byte [edx], bl

	pop	ebx
	ret
FUNCTION_END


FUNCTION pci_bios_find_device
	push	esi
	push	ebx

	mov	ecx, [esp + 4 * 3 + 4 * 0]
	mov	edx, [esp + 4 * 3 + 4 * 1]
	mov	esi, [esp + 4 * 3 + 4 * 2]

	PCI_BIOS32_CALL PCI_BIOS_FUNCTION_FIND_DEVICE

	mov	ecx, [esp + 4 * 3 + 4 * 3]
	mov	edx, [esp + 4 * 3 + 4 * 4]
	mov	byte [ecx], bh
	mov	byte [edx], bl

	pop	ebx
	pop	esi
	ret
FUNCTION_END
	
	
FUNCTION pci_bios_find_class
	push	esi
	push	ebx

	mov	ecx, [esp + 4 * 3 + 4 * 0]
	mov	esi, [esp + 4 * 3 + 4 * 1]

	PCI_BIOS32_CALL PCI_BIOS_FUNCTION_FIND_CLASS

	mov	ebx, [esp + 4 * 3 + 4 * 2]
	mov	ecx, [esp + 4 * 3 + 4 * 3]
	mov	byte [ebx], bh
	mov	byte [edx], bl
        
	pop	ebx
	pop	esi
	ret
FUNCTION_END


%macro PCI_BIOS32_READ_CONFIG 3
FUNCTION pci_bios_read_config_%1
	push	ebx

	mov	ebx, [esp + 4 * 2 + 4 * 1]
	mov	bh, [esp + 4 * 2 + 4 * 0]
	mov	di, [esp + 4 * 2 + 4 * 2]

	PCI_BIOS32_CALL PCI_BIOS_FUNCTION_READ_CONFIG_%1

	mov	ebx, [esp + 4 * 2 + 4 * 3]
	mov	%2 [ebx], %3

	pop	ebx
	ret
FUNCTION_END
%endmacro


PCI_BIOS32_READ_CONFIG 8, byte, cl
PCI_BIOS32_READ_CONFIG 16, word, cx
PCI_BIOS32_READ_CONFIG 32, dword, ecx


%macro PCI_BIOS32_WRITE_CONFIG 1
FUNCTION pci_bios_write_config_%1
	push	ebx

	mov	ebx, [esp + 4 * 2 + 4 * 1]
	mov	bh, [esp + 4 * 2 + 4 * 0]
	mov	di, [esp + 4 * 2 + 4 * 2]
	mov	ecx, [esp + 4 * 2 + 4 * 3]

	PCI_BIOS32_CALL PCI_BIOS_FUNCTION_WRITE_CONFIG_%1

	pop	ebx
	ret
FUNCTION_END
%endmacro


PCI_BIOS32_WRITE_CONFIG 8
PCI_BIOS32_WRITE_CONFIG 16
PCI_BIOS32_WRITE_CONFIG 32

