#ifndef __PERSISTENCE_MAPPING_H__
#define __PERSISTENCE_MAPPING_H__

#include <utility/types.h>
#include <utility/list.h>
#include <i386/page.h>

typedef enum
{
        MAPPING_FLAG_PRESENT   = I386_PAGE_ENTRY_PRESENT,
        MAPPING_FLAG_DIRTY     = I386_PAGE_ENTRY_DIRTY,
        MAPPING_FLAG_WRITEABLE = I386_PAGE_ENTRY_WRITEABLE,
        MAPPING_FLAG_LOCKED    = I386_PAGE_ENTRY_RESERVED_0,
} MappingFlag;

typedef struct
{
        ListNode         mappingList;
        i386PageEntry *  pageEntry;
        ListIterator     threadsWaiting;
} Mapping;

typedef struct
{
        ListNode  rangeList;
        Address   rangeBegin;
        Address   rangeEnd;
        Mapping * mappings;
} MappingRange;

extern Size      mapping_range_size         (Address, Address);
extern Bool      mapping_range_create       (MappingRange *, Address, Address);

extern void      mapping_range_insert       (MappingPageRange *);
extern MappingRange * mapping_range_find    (const Mapping *);

extern Mapping * mapping_find               (Address);
extern Mapping * mapping_allocate           (Address, Size, Address, FlagSet);
extern void      mapping_free               (Address, Size);
extern Address   mapping_virtual_address    (const Mapping *);
extern Address   mapping_physical_address   (const Mapping *);

#endif /* __PERSISTENCE_MAPPING_H__ */

