#include <interrupt/interrupt.h>
#include <interrupt/exception.h>
#include <scheduler/thread.h>
#include <scheduler/critical.h>

void
critical_enter (void)
{
   Thread * thread; 
 
   if (interrupt_level () > 0 || exception_level () > 0)
     return;

   thread = thread_current ();

   if (thread == NULL)
     return;

   if (thread -> criticalCount <= 0)
     interrupt_disable (INTERRUPT_NUMBER_ANY);

   ++ thread -> criticalCount;
}

void
critical_exit (void)
{
   Thread * thread; 
 
   if (interrupt_level () > 0 || exception_level () > 0)
     return;

   thread = thread_current ();

   if (thread == NULL ||
       thread -> criticalCount <= 0)
     return;

   -- thread -> criticalCount;
   
   if (thread -> criticalCount <= 0)
     interrupt_enable (INTERRUPT_NUMBER_ANY);
}

Size
critical_count (void)
{
   Thread * thread;

   if (interrupt_level () > 0 || exception_level () > 0)
     return ~0;
   
   thread = thread_current ();
   if (thread == NULL)
     return 0;

   return thread -> criticalCount;
}

