#ifndef __SCHEDULER_THREAD_H__
#define __SCHEDULER_THREAD_H__

#include <utility/types.h>
#include <utility/list.h>
#include <scheduler/registers.h>
#include <scheduler/spinlock.h>

struct Scheduler;

enum
{
	THREAD_STACK_SIZE = 16384
};

enum
{
	THREAD_YIELD_FREQUENCY = 10
};

enum
{
        THREAD_PRIORITY_IDLE      = 0,
        THREAD_PRIORITY_MINIMUM   = 1,
        THREAD_PRIORITY_MAXIMUM   = 10,
        THREAD_PRIORITY_EXCLUSIVE = 11
};

typedef Size    ThreadPriority;

enum
{
	THREAD_QUANTUM_MINIMUM = 1,
	THREAD_QUANTUM_MAXIMUM = 10,

	THREAD_QUANTUM_DELTA   = (THREAD_QUANTUM_MAXIMUM - THREAD_QUANTUM_MINIMUM) / (THREAD_PRIORITY_MAXIMUM - THREAD_PRIORITY_MINIMUM)
};

typedef Size	ThreadQuantum;

typedef enum
{
	THREAD_STATE_CREATE = 0,
	THREAD_STATE_READY,
	THREAD_STATE_RUNNING,
	THREAD_STATE_STOPPED,
	THREAD_STATE_WAITING_ON_MUTEX,
	THREAD_STATE_WAITING_ON_SEMAPHORE,
	THREAD_STATE_WAITING_ON_CONDITION
} ThreadState;

typedef void ThreadEnterFunction (void *);

typedef struct Thread
{
	ListNode	      threadList;
	struct Scheduler *    scheduler;
        ThreadRegisters	      registers;
	ThreadState	      state;
	ThreadQuantum	      quantumInitial;
	ThreadQuantum	      quantumCurrent;
	ThreadPriority	      priorityFloor;
	ThreadPriority	      priorityCeiling;
	ThreadPriority	      priorityCurrent;
        Size                  criticalCount;
	ThreadEnterFunction * enterFunction;
	void *		      enterData;
	uint8 *		      stack;
	Size		      stackSize;
	Bool		      stackAllocated;
} Thread;

extern Bool		thread_create		(Thread **, ThreadPriority, ThreadPriority, ThreadEnterFunction *, void *, void *, Size);
extern void		thread_destroy		(Thread *);

extern void		thread_block		(Thread *, ThreadState, Spinlock *, List *);
extern void		thread_stop		(Thread *);
extern void		thread_start		(Thread *);

extern void		thread_yield		(Thread *);
extern Thread *		thread_current		(void);

extern void *		thread_get_data		(Thread *);
extern void		thread_set_data		(Thread *, void *);

extern ThreadPriority	thread_get_priority	(Thread *);
extern Bool		thread_set_priority	(Thread *, ThreadPriority);

extern ThreadPriority	thread_get_priority_floor	(Thread *);
extern void		thread_set_priority_floor	(Thread *, ThreadPriority);

extern ThreadPriority	thread_get_priority_ceiling	(Thread *);
extern void		thread_set_priority_ceiling	(Thread *, ThreadPriority);

#endif /* __SCHEDULER_THREAD_H__ */

