#ifndef __STORAGE_STORAGE_H__
#define __STORAGE_STORAGE_H__

#include <utility/types.h>
#include <utility/list.h>
#include <scheduler/mutex.h>

struct StorageType;

typedef Size StorageBlock;
typedef Size StorageCapacity;

typedef void TransferRequestSuccessFunction (Storage *, StorageBlock, void *, Size, void *);
typedef void TransferRequestFailureFunction (Storage *, StorageBlock, void *, Size, void *);

typedef enum
{
        STORAGE_STATE_BUSY       = 1,
        STORAGE_STATE_READY      = 2,
        STORAGE_STATE_CANCEL     = 3
} StorageState;

typedef struct
{
        Mutex                   lock;
        struct StorageType *    type;
        Size                    blockSize;
        StorageCapacity         blockCapacity;
        StorageState            state;
        List                    transferRequests;
        void *                  data;
} Storage;

extern Bool     storage_create   (Storage **, struct StorageType *, Size, StorageCapacity, void *);
extern void     storage_destroy  (Storage *);

extern Bool     storage_cancel   (Storage *);
extern Bool     storage_flush    (Storage *);
extern Bool     storage_read     (Storage *, 
                                  StorageBlock, 
                                  void *, 
                                  Size,
                                  TransferRequestSuccessFunction *,
                                  void *,
                                  TransferRequestFailureFunction *,
                                  void *);
extern Bool     storage_write    (Storage *, 
                                  StorageBlock, 
                                  const void *, 
                                  Size,
                                  TransferRequestSuccessFunction *,
                                  void *,
                                  TransferRequestFailureFunction *,
                                  void *);

#endif /* __STORAGE_STORAGE_H__ */

