#include <i386/io.h>
#include <interrupt/interrupt.h>
#include <timer/pit.h>
#include <timer/timer.h>

static PITCounter       pitCount;
static InterruptHandler pitHandler;

static void
pit_interrupt_handler (InterruptNumber number, void * data)
{
   PITHandlerFunction * handlerFunction = (PITHandlerFunction *) data; 
   
   ++ pitCount;

   if (handlerFunction != NULL)
     (* handlerFunction) ();
}

void
pit_start (PITFrequency frequency, PITHandlerFunction * handlerFunction)
{
   uint16 ratio = PIT_FREQUENCY_BASE / frequency;

   i386_output_8 (PIT_REGISTER_COMMAND, 
                  PIT_COMMAND_SELECT_COUNTER_0 | 
                  PIT_COMMAND_READ_WRITE_LSB | 
                  PIT_COMMAND_READ_WRITE_MSB | 
                  PIT_MODE_RATE_GENERATOR);
   i386_output_8 (PIT_REGISTER_COUNTER_0, ratio & 0xFF);
   i386_output_8 (PIT_REGISTER_COUNTER_0, ratio >> 8);

   pitCount = 0;

   interrupt_handler_create (& pitHandler, 
                             0, 
                             pit_interrupt_handler, 
                             (void *) handlerFunction, 
                             0, 
                             "pit");
}

void
pit_stop (void)
{
   interrupt_handler_destroy (& pitHandler);
}

PITCounter
pit_count (void)
{
   return pitCount;
}

