%include "i386/boot.i"
%include "i386/manage.i"
%include "i386/general.i"

;;; Derived from Dolphin boot sector by Samuel Falvo. Dolphin licensing restrictions apply.

        bits 16
        org 0x7C00

_start:
        jmp     .beginLoad
        
.bootHeader:
        istruc i386BootHeader
                at i386BootHeader.magic,                dd 0
                at i386BootHeader.imageSize,            dd 0
                at i386BootHeader.loadAddress,          dd 0
                at i386BootHeader.startAddress,         dd 0
                at i386BootHeader.informationAddress,   dd 0
        iend

.beginLoad:
        mov     ax, cs
        mov     ds, ax
        mov     es, ax
       
        push    word I386_EFLAGS_FLAG_RESERVED
        popf

        mov     ax, [.bootHeader + i386BootHeader.imageSize]
        mov     bx, [.bootHeader + i386BootHeader.imageSize + 2]
        add     ax, (1 << 9) - 1
        adc     bx, 0
        shr     ax, 9
        shl     bx, 16 - 9
        add     ax, bx

        mov     dx, 0x10000 / 16
        mov     es, dx
        xor     bx, bx
        mov     cx, 1

.readNextSector:
        call    .readSector
        add     dx, 512 / 16
        mov     es, dx
        inc     cx
        cmp     cx, ax
        jbe     .readNextSector

.initializeGDT:
        xor     ax, ax
        mov     ds, ax
        mov     es, ax

        mov     di, I386_GDT_OFFSET
        mov     cx, I386_GDT_LIMIT 
        xor     ax, ax
        rep     stosb

        mov     di, I386_GDT_OFFSET + I386_SELECTOR_TSS
        mov     si, globalDescriptorTable
        mov     cx, I386_SELECTOR_USER_DATA + 8 - I386_SELECTOR_TSS
        rep     movsb

        mov     bp, sp
        sub     bp, 8
        mov     [bp], word I386_GDT_LIMIT
        mov     [bp + 2], word I386_GDT_OFFSET
        mov     [bp + 4], ax
        lgdt    [bp]

.enableA20Gate:
        call    .keyboardWaitForCommand
        mov     al, 0xD0
        out     0x64, al
       
        call    .keyboardWaitForData
        in      al, 0x60
        or      al, 2
        mov     dl, al
       
        call    .keyboardWaitForCommand
        mov     al, 0xD1
        out     0x64, al

        call    .keyboardWaitForCommand
        mov     al, dl 
        out     0x60, al

        call    .keyboardWaitForCommand

.getMemorySize:
        mov     ah, 0x88
        int     0x15
        mov     bx, ax

.enterProtectedMode:
        mov     eax, I386_CR0_PROTECTION_ENABLE
        mov     cr0, eax

        jmp     I386_SELECTOR_SYSTEM_CODE : .loadCS


        bits 32
.loadCS:
        mov     ax, I386_SELECTOR_SYSTEM_DATA
        mov     ds, ax
        mov     es, ax
        mov     fs, ax
        mov     gs, ax
        mov     ss, ax
        
        mov     edi, [.bootHeader + i386BootHeader.loadAddress]
        mov     esi, 0x10000
        mov     ecx, [.bootHeader + i386BootHeader.imageSize]
        shr     ecx, 2
        rep     movsd
        
        mov     ecx, [.bootHeader + i386BootHeader.imageSize]
        and     ecx, 3
        rep     movsb

        mov     edi, [.bootHeader + i386BootHeader.informationAddress]
        mov     esi, .bootHeader
        mov     ecx, i386BootInformation.memorySize
        rep     movsb

        and     ebx, 0xFFFF
        shr     ebx, 2
        add     ebx, 0x100000 / 4096
        mov     [edi], ebx

        jmp     dword [.bootHeader + i386BootHeader.startAddress]
     
 
        bits 16
.readSector:
        pushf
        pusha

        mov     ax, cx
        xor     dx, dx
        mov     cx, 18
        div     cx
        mov     cl, dl
        inc     cl
        xor     dh, dh
        shr     ax, 1
        rcl     dh, 1
        mov     ch, al
        mov     al, 1
        mov     ah, 2
        mov     dl, 0
        int     0x13
        jc      .readError

        popa
        popf
        ret

.readError:
        hlt
        jmp     .readError


.keyboardWaitForData:
        in      al, 0x64
        test    al, 1
        jz      .keyboardWaitForData
        ret

        
.keyboardWaitForCommand:
        in      al, 0x64
        test    al, 2
        jnz     .keyboardWaitForCommand
        ret


globalDescriptorTable:
; TSS (0x0018)
        dw      I386_TSS_LIMIT - 1
        dw      I386_TSS_OFFSET
        db      0
        db      I386_DESCRIPTOR_ATTRIBUTE_TYPE_TSS | I386_DESCRIPTOR_ATTRIBUTE_PRESENT
        db      0
        db      0

; system code segment (0x0020)
        dw      0xFFFF
        dw      0x0000
        db      0
        db      I386_DESCRIPTOR_ATTRIBUTE_READ_ENABLE | I386_DESCRIPTOR_ATTRIBUTE_NON_SYSTEM | I386_DESCRIPTOR_ATTRIBUTE_CODE | I386_DESCRIPTOR_ATTRIBUTE_PRESENT
        db      0xF | I386_DESCRIPTOR_ATTRIBUTE_PAGE_GRANULARITY | I386_DESCRIPTOR_ATTRIBUTE_32_BIT
        db      0

; system data segment (0x0028)
        dw      0xFFFF
        dw      0x0000
        db      0
        db      I386_DESCRIPTOR_ATTRIBUTE_READ_ENABLE | I386_DESCRIPTOR_ATTRIBUTE_NON_SYSTEM | I386_DESCRIPTOR_ATTRIBUTE_PRESENT
        db      0xF | I386_DESCRIPTOR_ATTRIBUTE_PAGE_GRANULARITY | I386_DESCRIPTOR_ATTRIBUTE_32_BIT
        db      0

; user code segment (0x0030)
        dw      0xFFFF
        dw      0x0000
        db      0
        db      I386_DESCRIPTOR_ATTRIBUTE_READ_ENABLE | I386_DESCRIPTOR_ATTRIBUTE_NON_SYSTEM | I386_DESCRIPTOR_ATTRIBUTE_CODE | I386_DESCRIPTOR_ATTRIBUTE_DPL_3 | I386_DESCRIPTOR_ATTRIBUTE_PRESENT
        db      0xF | I386_DESCRIPTOR_ATTRIBUTE_PAGE_GRANULARITY | I386_DESCRIPTOR_ATTRIBUTE_32_BIT
        db      0

; user data segment (0x0038)
        dw      0xFFFF
        dw      0x0000
        db      0
        db      I386_DESCRIPTOR_ATTRIBUTE_READ_ENABLE | I386_DESCRIPTOR_ATTRIBUTE_NON_SYSTEM | I386_DESCRIPTOR_ATTRIBUTE_DPL_3 | I386_DESCRIPTOR_ATTRIBUTE_PRESENT
        db      0xF | I386_DESCRIPTOR_ATTRIBUTE_PAGE_GRANULARITY | I386_DESCRIPTOR_ATTRIBUTE_32_BIT
        db      0

.bootMagic:
        times 510-($-$$) db 0
        dw 0xAA55

