#include <stdlib.h>
#include "symbol.h"
#include "export.h"
#include "import.h"
#include "component.h"

Component *
component_create (const char * name)
{
        Component * component = (Component *) malloc (sizeof (Component));

        if (component == NULL)
          return NULL;

        component -> name = name;
        
        list_clear (& component -> exports);
        list_clear (& component -> imports);
        list_clear (& component -> constructors);
        list_clear (& component -> destructors);

        return component;
}

void
component_destroy (Component * component)
{
        while (list_empty (& component -> constructors) == 0)
           symbol_destroy ((Symbol *) list_front (& component -> constructors));

        while (list_empty (& component -> destructors) == 0)
           symbol_destroy ((Symbol *) list_front (& component -> destructors));

        while (list_empty (& component -> exports) == 0)
           export_destroy ((Export *) list_front (& component -> exports));

        while (list_empty (& component -> imports) == 0)
           import_destroy ((Import *) list_front (& component -> imports));

        free (component);
}

int
component_include (Component * dst, Component * src)
{
        ListIterator position;

        for (position = list_begin (& src -> exports);
             position != list_end (& src -> exports);
             position = list_next (position))
        {
            Export * export = (Export *) position;

            if (export_create (dst, export -> interface) == NULL)
              return -1;
        }

        for (position = list_begin (& src -> imports);
             position != list_end (& src -> imports);
             position = list_next (position))
        {
            Import * import = (Import *) position;

            if (import_create (dst, import -> interface) == NULL)
              return -1;
        }

        for (position = list_begin (& src -> constructors);
             position != list_end (& src -> constructors);
             position = list_next (position))
        {
            Symbol * constructor = (Symbol *) position;

            if (symbol_create (& dst -> constructors, constructor -> name, constructor -> alias) == NULL)
              return -1;
        }

        for (position = list_previous (list_end (& src -> destructors));
             position != list_end (& src -> destructors);
             position = list_previous (position))
        {
            Symbol * destructor = (Symbol *) position;

            destructor = symbol_create (& dst -> destructors, destructor -> name, destructor -> alias);
            
            if (destructor == NULL)
              return -1;

            list_move_to_front (& dst -> destructors, & destructor -> symbolList);
        }
        
        return 0;
}

void
component_prune (Component * component)
{
        ListIterator position;

        for (position = list_begin (& component -> exports);
             position != list_end (& component -> exports);
             position = list_next (position))
        {
            Export * export = (Export *) position;
            Import * import = import_find (component, export -> interface);

            if (import != NULL)
              import_destroy (import);
        }
}

        
