#include <stdio.h>
#include <string.h>
#include "pcom.h"
#include "error.h"
#include "llexer.h"
#include "symbol.h"
#include "import.h"
#include "export.h"
#include "interface.h"
#include "component.h"
#include "pidl.h"

int
pidl_create (PIDL * pidl)
{
        if (namespace_create (& pidl -> namespace) != 0)
          return -1;

        pidl -> directories    = NULL;
        pidl -> directoryCount = 0;

        return 0;
}

void
pidl_destroy (PIDL * pidl)
{
        if (pidl -> directoryCount > 0 &&
            pidl -> directories != NULL)
          free (pidl -> directories);

        namespace_destroy (& pidl -> namespace);
}

int
pidl_add_directory (PIDL * pidl, const char * directory)
{
        const char ** newDirectories;

        newDirectories = (const char **) realloc (pidl -> directories, 
                                                  (pidl -> directoryCount + 1) * sizeof (const char *));

        if (newDirectories == NULL)
          return -1;

        newDirectories [pidl -> directoryCount] = directory;

        pidl -> directories = newDirectories;
       
        ++ pidl -> directoryCount;

        return 0;
}

int
pidl_add_interface (PIDL * pidl, Value * value)
{
        ListIterator position;
        const char * name;
        Interface * interface;
        Definition * definition;

        if (value_match (value, "('interface' s r*)", & name, & position) != 0)
          pidl_error ("Bad interface command");

        definition = namespace_insert (& pidl -> namespace, name);

        if (definition -> interface != NULL)
        {
           interface = definition -> interface;
        }
        else
        {
           interface = interface_create (definition -> name);
           if (interface == NULL)
             return -1;

           definition -> interface = interface;
        }

        while (position != list_end (& value -> datum.list))
        {
            Value * command = (Value *) position;
            const char * symbolName = NULL,
                       * symbolAlias = NULL,
                       * commandName;

            if (value_match (command, "('function' s)", & symbolName) != 0 &&
                value_match (command, "('function-as' s s)", & symbolName, & symbolAlias) != 0 &&
                value_match (command, "('data' s)", & symbolName) != 0 &&
                value_match (command, "('data-as' s s)", & symbolName, & symbolAlias) != 0)
            {
               if (value_match (command, "(s *)", & commandName) == 0)
                 pidl_error ("Bad interface %s sub-command %s", name, commandName);
               else
                 pidl_error ("Bad interface %s sub-command", name);
            }

            if (symbolAlias == NULL)
              symbolAlias = symbolName;

            symbolName  = namespace_insert (& pidl -> namespace, symbolName) -> name;
            symbolAlias = namespace_insert (& pidl -> namespace, symbolAlias) -> name;

            if (symbol_create (& interface -> symbols, symbolName, symbolAlias) == NULL)
              return -1;

            position = list_next (position);
        }

        return 0;
}

int
pidl_add_component (PIDL * pidl, Value * value)
{
        ListIterator position;
        const char * name;
        Component * component;
        Definition * definition;

        if (value_match (value, "('component' s r*)", & name, & position) != 0)
          pidl_error ("Bad component command");

        definition = namespace_insert (& pidl -> namespace, name);

        if (definition -> component != NULL)
        {
           component = definition -> component;
        }
        else
        {
           component = component_create (definition -> name);
           if (component == NULL)
             return -1;

           definition -> component = component;
        }

        while (position != list_end (& value -> datum.list))
        {
              Value * command = (Value *) position;
              const char * interfaceName = NULL,
                         * symbolName = NULL,
                         * symbolAlias = NULL,
                         * componentName = NULL,
                         * commandName;

              if (value_match (command, "('import' s)", & interfaceName) == 0)
              {
                  definition = namespace_find (& pidl -> namespace, interfaceName);

                  if (definition == NULL ||
                      definition -> interface == NULL ||
                      import_create (component, definition -> interface) == NULL)
                    return -1;
              }
              else
              if (value_match (command, "('export' s)", & interfaceName) == 0)
              {
                  definition = namespace_find (& pidl -> namespace, interfaceName);

                  if (definition == NULL ||
                      definition -> interface == NULL ||
                      export_create (component, definition -> interface) == NULL)
                    return -1;
              }
              else
              if (value_match (command, "('constructor' s)", & symbolName) == 0 ||
                  value_match (command, "('constructor-as' s s)", & symbolName, & symbolAlias) == 0)
              { 
                  if (symbolAlias == NULL)
                    symbolAlias = symbolName;

                  symbolName  = namespace_insert (& pidl -> namespace, symbolName) -> name;
                  symbolAlias = namespace_insert (& pidl -> namespace, symbolAlias) -> name;

                  if (symbol_create (& component -> constructors, symbolName, symbolAlias) == NULL)
                    return -1;
              }
              else
              if (value_match (command, "('destructor' s)", & symbolName) == 0 ||
                  value_match (command, "('destructor-as' s s)", & symbolName, & symbolAlias) == 0)
              {
                  if (symbolAlias == NULL)
                    symbolAlias = symbolName;

                  symbolName  = namespace_insert (& pidl -> namespace, symbolName) -> name;
                  symbolAlias = namespace_insert (& pidl -> namespace, symbolAlias) -> name;

                  if (symbol_create (& component -> destructors, symbolName, symbolAlias) == NULL)
                    return -1;
              }
              else
              if (value_match (command, "('include' s)", & componentName) == 0)
              {
                 definition = namespace_find (& pidl -> namespace, componentName);

                 if (definition == NULL ||
                     definition -> component == NULL ||
                     component_include (component, definition -> component) != 0)
                   return -1;
              }
              else
              if (value_match (command, "(s *)", & commandName) == 0)
                pidl_error ("Bad component %s sub-command %s", name, commandName);
              else
                pidl_error ("Bad component %s sub-command");

              position = list_next (position);
        }                  

        return 0;
}

int
pidl_include_file (PIDL * pidl, Value * value)
{
        FILE * file;
        const char * fileName;

        if (value_match (value, "('include' s)", & fileName) != 0)
          pidl_error ("Bad include command");

        pidl_message ("Including file %s", fileName);

        file = pidl_find_file (pidl, fileName);
        if (file == NULL)
          return -1;

        if (pidl_read (pidl, file, 0) != 0)
        {
           fclose (file);

           pidl_error ("%s: Failed to read include file", fileName);
        }

        fclose (file);

        return 0;
}

int
pidl_import_file (PIDL * pidl, Value * value)
{
        FILE * file;
        const char * fileName;

        if (value_match (value, "('import' s)", & fileName) != 0)
          pidl_error ("Bad import command");

        pidl_message ("Importing file %s", fileName);

        file = pidl_find_file (pidl, fileName);
        if (file == NULL)
          return -1;

        if (pidl_read (pidl, file, 1) != 0)
        {
           fclose (file);

           pidl_error ("%s: Failed to read import file", fileName);
        }

        fclose (file);

        return 0;
}

FILE *
pidl_find_file (PIDL * pidl, const char * name)
{
        static char buffer [1024];
 
        FILE * file;
        
        const char ** directory;
        
        size_t nameLength = strlen (name);

        for (directory = pidl -> directories;
             directory < & pidl -> directories [pidl -> directoryCount];
             ++ directory)
        {
           size_t directoryLength = strlen (* directory); 

           if (directoryLength + nameLength >= sizeof (buffer))
             return NULL;

           strcpy (buffer, * directory);
           strcpy (& buffer [directoryLength], name);

           file = fopen (buffer, "r");   

           if (file != NULL)
             return file;
        }

        return fopen (name, "r");
}

int
pidl_read (PIDL * pidl, FILE * file, int onlyInterfaces)
{
        LLexer lexer;
        Value * command;

        llexer_create (& lexer, file);

        for (command = value_read (& lexer);
             command != NULL;
             command = value_read (& lexer))
        {
            const char * commandName;

            if (value_match (command, "(s *)", & commandName) != 0)
              return -1;

            if (strcmp (commandName, "interface") == 0)
            {
                if (pidl_add_interface (pidl, command) != 0)
                  return -1;
            }
            else
            if (strcmp (commandName, "component") == 0)
            {
                if (onlyInterfaces == 0 &&
                    pidl_add_component (pidl, command) != 0)
                  return -1;
            }
            else
            if (strcmp (commandName, "import") == 0)
            {
                if (onlyInterfaces == 0 &&
                    pidl_import_file (pidl, command) != 0)
                  return -1;
            }
            else
            if (strcmp (commandName, "include") == 0)
            {
                if (onlyInterfaces == 0 &&
                    pidl_include_file (pidl, command) != 0)
                  return -1;
            }
            else
              pidl_error ("Unknown command %s", commandName);
        } 
        
        return 0;
}

int
pidl_write_symbols (PIDL * pidl, List * symbols, FILE * file)
{
        ListIterator position;

        for (position = list_begin (symbols);
             position != list_end (symbols);
             position = list_next (position))
        {
            Symbol * symbol = (Symbol *) position;
            PComSymbol pcomSymbol;

            pcomSymbol.name  = (PComString) namespace_find (& pidl -> namespace, symbol -> name) -> offset;
            pcomSymbol.alias = (PComString) namespace_find (& pidl -> namespace, symbol -> alias) -> offset;

            if (fwrite (& pcomSymbol, sizeof (PComSymbol), 1, file) != 1)
              return -1;
        }

        return 0;
}

int
pidl_write (PIDL * pidl, Component * component, FILE * file)
{
        ListIterator position;
        PComHeader header;
        PComSection * sections,
                    * currentSection;
        PComAddress offset;

        memcpy (header.signature, PCOM_SIGNATURE, PCOM_SIGNATURE_LENGTH);

        header.version        = PCOM_VERSION;
        header.stringsSection = 0;
       
        if (list_empty (& component -> constructors) == 0)
          header.constructorsSection = 1;
        else
          header.constructorsSection = PCOM_SECTION_NUMBER_NULL;
       
        if (list_empty (& component -> destructors) == 0)
        {
           if (header.constructorsSection != PCOM_SECTION_NUMBER_NULL)
             header.destructorsSection = 2;
           else
             header.destructorsSection = 1;
        }
        else
          header.destructorsSection = PCOM_SECTION_NUMBER_NULL;

        header.sectionCount = list_size (& component -> exports) +
                              list_size (& component -> imports) +
                              (list_empty (& component -> constructors) ? 0 : 1) +
                              (list_empty (& component -> destructors) ? 0 : 1) + 
                              1;

        sections = (PComSection *) calloc (header.sectionCount, sizeof (PComSection));

        if (sections == NULL)
          return -1;

        offset = sizeof (PComHeader) + header.sectionCount * sizeof (PComSection);
        
        sections -> name   = PCOM_STRING_NULL;
        sections -> type   = PCOM_SECTION_TYPE_STRINGS;
        sections -> offset = offset;
        sections -> size   = (PComSize) namespace_assign_offsets (& pidl -> namespace);

        header.name = (PComString) namespace_find (& pidl -> namespace, component -> name) -> offset;
        
        offset += sections -> size;
       
        currentSection = & sections [1];
 
        if (header.constructorsSection != PCOM_SECTION_NUMBER_NULL)
        {
           currentSection -> name        = PCOM_STRING_NULL;
           currentSection -> type        = PCOM_SECTION_TYPE_CONSTRUCTORS;
           currentSection -> offset      = offset;
           currentSection -> symbolCount = list_size (& component -> constructors);
           currentSection -> size        = currentSection -> symbolCount * sizeof (PComSymbol);

           offset += currentSection -> size;

           ++ currentSection;
        }
       
        if (header.destructorsSection != PCOM_SECTION_NUMBER_NULL)
        {
           currentSection -> name        = PCOM_STRING_NULL;
           currentSection -> type        = PCOM_SECTION_TYPE_DESTRUCTORS;
           currentSection -> offset      = offset;
           currentSection -> symbolCount = list_size (& component -> destructors);
           currentSection -> size        = currentSection -> symbolCount * sizeof (PComSymbol);

           offset += currentSection -> size;

           ++ currentSection;
        }

        for (position = list_begin (& component -> exports);
             position != list_end (& component -> exports);
             position = list_next (position))
        {
            Export * export = (Export *) position;

            currentSection -> name        = (PComString) namespace_find (& pidl -> namespace, 
                                                                         export -> interface -> name) -> offset;
            currentSection -> type        = PCOM_SECTION_TYPE_EXPORTS;
            currentSection -> offset      = offset;
            currentSection -> symbolCount = list_size (& export -> interface -> symbols);
            currentSection -> size        = currentSection -> symbolCount * sizeof (PComSymbol);
              
            offset += list_size (& export -> interface -> symbols) * sizeof (PComSymbol);

            ++ currentSection;
        }

        for (position = list_begin (& component -> imports);
             position != list_end (& component -> imports);
             position = list_next (position))
        {
            Import * import = (Import *) position;

            currentSection -> name        = (PComString) namespace_find (& pidl -> namespace, 
                                                                         import -> interface -> name) -> offset;
            currentSection -> type        = PCOM_SECTION_TYPE_IMPORTS;
            currentSection -> offset      = offset;
            currentSection -> symbolCount = list_size (& import -> interface -> symbols);
            currentSection -> size        = currentSection -> symbolCount * sizeof (PComSymbol);

            offset += list_size (& import -> interface -> symbols) * sizeof (PComSymbol);

            ++ currentSection;
        }

        header.size = offset;

        if (fwrite (& header, sizeof (PComHeader), 1, file) != 1 ||
            fwrite (sections, header.sectionCount * sizeof (PComSection), 1, file) != 1 ||
            namespace_write (& pidl -> namespace, file) != 0 ||
            pidl_write_symbols (pidl, & component -> constructors, file) != 0 ||
            pidl_write_symbols (pidl, & component -> destructors, file) != 0)
        {
           free (sections);

           return -1;
        }

        free (sections);

        for (position = list_begin (& component -> exports);
             position != list_end (& component -> exports);
             position = list_next (position))
        {
            Export * export = (Export *) position;
            
            if (pidl_write_symbols (pidl, & export -> interface -> symbols, file) != 0)
              return -1;
        }

        for (position = list_begin (& component -> imports);
             position != list_end (& component -> imports);
             position = list_next (position))
        {
            Import * import = (Import *) position;

            if (pidl_write_symbols (pidl, & import -> interface -> symbols, file) != 0)
              return -1;
        }

        return 0;
}
         
            
